<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: MapController.php 3556 2014-05-28 10:34:33Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Default
 * @copyright  Copyright (c) 2012 VEC
 */

class MapController extends Zend_Controller_Action
{

    public function indexAction()
    {
        //Reset settings
        $settings = Zend_Registry::get('settings');
        if(! $this->_request->getParam('type')) {
            $settings->historyId = NULL;
            $settings->editingActive = false;
            $settings->historyDateBegin = NULL;
            $settings->historyDateEnd = NULL;
            $settings->querybuilder = NULL;
        }
    }

    public function zoomtolayerAction()
    {
        //
        $config = Zend_Registry::get('config');

        $result['replyCode'] = 200;
        $result['extents'] = array();

        //
        $request = $this->getRequest();
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        $layersTable = new Giscuit_Db_Table_Layers();

        //
        $ids = $request->getParam('ids');
        $ids = explode(',', $ids);
        $ids = array_map('intval', $ids);

        if(! empty($userId)) {
            $mapOptions = array(
                'orderDirection' => 'asc' ,
                'whereActive' => 't' ,
                'withAcl' => true ,
            );
        } else {
            $mapOptions = array(
                'orderDirection' => 'asc' ,
                'wherePublish' => 't' ,
                'whereActive' => 't' ,
                'withPublish' => true
            );
        }
        $rows = $layersTable->getItems($mapOptions);

        //
        foreach($rows as $key => $val) {
            //
            if(! in_array($val['id'], $ids)) {
                continue;
            }

            $className = $tableName = 'Giscuit_Map_Layer_' . $val['class'];

            //Vector
            if (class_exists($className) && is_subclass_of($className, 'Giscuit_Map_Layer_Vector')) {
                $mapOptions = array(
                    'whereActive' => 't' ,
                    'withAcl' => true ,
                    'id' => $val['id']
                );
                $table = Giscuit_Db_Table_Layers::getLayerTableById($val['id'], $mapOptions);

                if($table) {
                    $result['extents'][] = $table->getExtent();
                } else {
                    $extent = $config->map->extent;
                    $result['extents'][] = "{$extent->min->x},{$extent->min->y},{$extent->max->x},{$extent->max->y}";
                }

            //Raster
            } else if (class_exists($className) && is_subclass_of($className, 'Giscuit_Map_Layer_Raster')) {
                if(realpath($val['data']) === false) {
                    $path = GT_RASTER_PATH . DIR_SEP . $val['data'];
                } else {
                    $path = $val['data'];
                }

                if(! empty($val['srid']) && $val['srid'] != $config->map->srid) {

                } else {
                    $cmd = " -nogcp -nomd -norat -noct {$path}";
                    $cmdResult = Giscuit_Exec::exec('gdalinfo', $cmd, GT_GDAL_BIN_PATH);

                    if(! $cmdResult['error']) {
                        preg_match_all("/(Left|Right)\s+\(  (\d+\.\d+),  (\d+\.\d+)\)/", $cmdResult['output'], $matches);
                        $minx = min($matches[2]);
                        $maxx = max($matches[2]);
                        $miny = min($matches[3]);
                        $maxy = max($matches[3]);

                        $result['extents'][] = implode(',', array($minx, $miny, $maxx, $maxy));
                    }
                }

            }
        }

        $this->_helper->json($result);
        
    }
}