<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id$
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Tool
 * @copyright  Copyright (c) 2012 VEC
 */

class Tool_QuerybuilderController extends Zend_Controller_Action
{
    public $timeout = 10000;
    
    public $datatableRecordsLimit = 100;
    
    public $columnValuesLimit = 100;
    
    public $operators = array(
        "=" , 
        "!=" , 
        "<" , 
        "<=" , 
        ">" , 
        ">=" , 
        "IS NULL" , 
        "IS NOT NULL" , 
        "ILIKE" , 
        "NOT ILIKE",
        "IN" ,
        "NOT IN" ,
    );
    
    public $functs = array(
        "ST_Area" ,
        "ST_Length" ,
        "ST_Perimeter" ,
    
        "ST_Intersects" ,
        "ST_Disjoint" ,
            
        "ST_DWithin" ,
    
        "ST_Contains" ,
        //"NOT ST_Contains" ,
            
        "ST_Crosses" , 
        //"NOT ST_Crosses" ,
     
        "ST_Equals" ,
        //"NOT ST_Equals" ,
     
        //"ST_IsValid" ,
        //"NOT ST_IsValid" ,
    
        "ST_Overlaps" ,
        //"NOT ST_Overlaps" ,
     
        "ST_Touches" ,
        //"NOT ST_Touches" ,
    
        //"ST_Within" ,
        //"NOT ST_Within" ,
    );
    
    public $measurementOperators = array(
        "=" , 
        "!=" , 
        "<" , 
        "<=" , 
        ">" , 
        ">=" 
    );
    
    public $measurementFuncts = array(
        "ST_Area" ,
        "ST_Length" ,
        "ST_Perimeter"
    );
    
    public $formats = array(
        'csv' => 'CSV' ,
        'kml' => 'KML' ,
        'shp' => 'ESRI Shapefile' ,
        'gml' => 'GML' ,
        'tab' => 'MapInfo File' ,
    );
    
    protected $_mapOptions = array(
        'orderDirection' => 'asc' , 
        'whereActive' => 't' , 
        'withLayerSettings' => true , 
        'withAcl' => true , 
        'withFields' => true ,
        'withGroups' => true
    );
    
    public function indexAction()
    {
        $settings = Zend_Registry::get('settings');
                
        $result['replyCode'] = 200;
        
        if(isset($settings->querybuilder)) {
            $settings->querybuilder = NULL;
        }
        
        $layersTable = new Giscuit_Db_Table_Layers();
        $layers = $layersTable->getItems($this->_mapOptions);

        foreach($layers as $layer) {
            $className = 'Giscuit_Map_Layer_' . $layer['class'];
            
            if(! @class_exists($className) 
                || ! is_subclass_of($className, 'Giscuit_Map_Layer_Vector')) {
                    
                continue;
            }
            
            //Table
            $tableName = 'Giscuit_Db_Table_Layer_' . $layer['class'];
            $tableOptions = array(
                'name' => $layer['table']
            );
            $table = new $tableName($tableOptions);
            $table->setSettings($layer);
            
            //Path
            $pathValue = '';
            $path = $layer['path'];
            
            if (! empty($path) && $path != '/') {
                $path = trim($path, '/');
                $path = explode('/', $path);
                if (count($path) > 1) {
                    foreach ($path as $key => $value) {
                        $path[$key] = $layers[$path[$key]]['name'];
                    }
                    unset($path[count($path) - 1]);
                    
                    $pathValue = implode('/', $path) . '/';
                }
            }                     
            $layerResult = array(
                'id' => $layer['id'] ,
                'name' => $layer['name'] ,
                'path' => $pathValue ,
                'fields' => array() ,
            );
            
            $fields = $table->getPublicFields();
            
            foreach($fields as $field) {
                $fieldResult = array(
                    'name' => $field['name'] ,
                );
                $layerResult['fields'][] = $fieldResult;
            }
            
            //Add geomtry field
            $layerResult['fields'][] = array(
                'name' => 'the_geom'
            );
            
            $result['records'][] = $layerResult;
        }
        
        $result['limit'] = $this->datatableRecordsLimit;
        $result['operators'] = $this->operators;
        $result['functs'] = $this->functs;
        $result['measurementFuncts'] = $this->measurementFuncts;
        $result['formats'] = $this->formats;
        $result['measurementOperators'] = $this->measurementOperators;
                
        $this->_helper->json($result);
    }
    
    
    
    public function submitAction ()
    {
        $settings = Zend_Registry::get('settings');
        $db = Zend_Registry::get('db');
        $translate = Zend_Registry::get('Zend_Translate');
        
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        $id = intval($request->getParam('id'));
        
        $countText = '';
        
        try {
            $data = Zend_Json::decode($request->getParam('data'));
            $match = Zend_Json::decode($request->getParam('match'));
        } catch (Exception $e) {
            $result['error'] = $e->getMessage();
            $this->_helper->json($result);
            return;
        }
        
        $save = $request->getParam('save');
        if($save != 't') {
            $save = 'f';
        }
        
        $tables = $fields = $validData = array();
        
        if(isset($settings->querybuilder)) {
            $settings->querybuilder = NULL;
        }
        
        $layersTable = new Giscuit_Db_Table_Layers();
        $layers = $layersTable->getItems($this->_mapOptions);
        
        foreach($layers as $layer) {
            $className = 'Giscuit_Map_Layer_' . $layer['class'];
            
            if(! @class_exists($className) 
                || ! is_subclass_of($className, 'Giscuit_Map_Layer_Vector')) {
                    
                continue;
            }
           
            //Tables
            $tableName = 'Giscuit_Db_Table_Layer_' . $layer['class'];
            $tableOptions = array(
                'name' => $layer['table']
            );
            $table = new $tableName($tableOptions);
            $table->setSettings($layer);
            $tables[$layer['id']] = $table;

            //Fields
            $tmpFields = $table->getPublicFields();
            $tmpFields['the_geom'] = array(
                'name' => 'the_geom'
            );
            $fields[$layer['id']]= $tmpFields;
        }
        
        foreach($layers as $layer) {
            if(empty($data[$layer['id']])) {
                continue;
            }
            
            $conditions = $data[$layer['id']];
            
            foreach ($conditions as $line) {
                //Check for empty values
                if(empty($line['field'])) {
                    continue;
                }
                                
                $line['field'] = explode(',', $line['field']);
                
                if(count($line['field']) != 2) {
                    throw new Exception("Invalid field format ({$line['field']})");
                }
                 
                if(empty($layers[$line['field'][0]])) {
                    throw new Exception("Invalid layer id ({$line['field'][0]})");
                }
                
                $conditionLayerId = $line['field'][0];
                    
                //Validate operator
                if(! empty($line['operator']) && 
                    ! in_array($line['operator'], $this->operators) &&
                    ! in_array($line['operator'], array_keys($layers))
                    ) {
                        
                    throw new Giscuit_Exception("Invalid operator ({$line['operator']})");
                }
                
                //Validate function
                if(! empty($line['funct']) && ! in_array($line['funct'], $this->functs)) {
                    throw new Giscuit_Exception("Invalid function ({$line['funct']})");
                }
                    
                //Set field
                if(! in_array($line['field'][1], array_keys($fields[$conditionLayerId]))) {
                    throw new Giscuit_Exception("Invalid field ({$line['field'][1]})");
                }
                
                $line['field'] = $fields[$conditionLayerId][$line['field'][1]];
                
                //FIXME: Convert to string for JSON encode
                if(! empty($line['field']["expr"])) {
                    $line['field']["expr"] = (string) $line['field']["expr"];
                }
                    
                //Set table
                $line['table'] = $layers[$conditionLayerId]['table'];
                    
                //Set geometry table
                if($line['field']['name'] == 'the_geom') {
                    //Validate layer id
                    if($line['funct'] == 'ST_DWithin') {
                        if(! empty($line['operator'])) {
                            $line['geometryTable'] = $layers[$line['operator']]['table'];
                        }
                    } else {
                        if(! empty($layers[$line['value0']])) {
                            $line['geometryTable'] = $layers[$line['value0']]['table'];
                        }
                    }
                }

                unset($line['action']);
                $validData[$layer['id']]['conditions'][] = $line;
                  
            }
            
            //Match
            if(isset($validData[$layer['id']]['conditions']) 
                && count($validData[$layer['id']]['conditions']) > 0) {
                
                if(! empty($match[$layer['id']])) {
                    $validData[$layer['id']]['match'] = $match[$layer['id']];
                } else {
                   $validData[$layer['id']]['match'] = 1;
                }
            } 
        }
                
        $validData['save'] = $save;
        
        //Fields
        $result['fields'] = array();
        foreach ($tables[$id]->getPublicFields() as $field) {
            $result['fields'][] = $field['name'];
        }
       
        //Scale
        $result['scale'] = $layers[$id]['search_scale'];
        
        //Layer type
        $layerType = NULL;
        $className = 'Giscuit_Map_Layer_' . $layers[$id]['class'];
        if (is_subclass_of($className, 'Giscuit_Map_Layer_Point')) {
            $layerType = 'point';
        } else if (is_subclass_of($className, 'Giscuit_Map_Layer_Line')) {
            $layerType = 'line';
        } else if (is_subclass_of($className, 'Giscuit_Map_Layer_Polygon')) {
            $layerType = 'polygon';
        }
                
        $db->query("SET statement_timeout TO {$this->timeout};");
        try {
            //Test selects 
            foreach ($validData as $key => $value) {
                if(! isset($tables[$key])) {
                    continue;
                }
                
                //
                if($layerType == 'point') {
                    $countStmt = 'count';
                } else if ($layerType == 'line') {
                    $countStmt = 'length';
                } else if ($layerType == 'polygon') {
                    $countStmt = 'area';
                }
                
                //                
                $countResult = $db->fetchOne($tables[$key]->getSelect($countStmt, array(
                    'querybuilder' => $validData , 
                )));
                
                //
                
                if($layerType == 'point') {
                    $countText = $translate->_('totalAmount') . ': ' . $countResult;
                } else if ($layerType == 'line') {
                    $countText = $translate->_('totalLength') . ': ' . round($countResult, 2);
                } else if ($layerType == 'polygon') {
                    $countText = $translate->_('totalArea') . ': ' . round($countResult, 2);
                }
            }
            
            //
            $db->query("SET statement_timeout TO DEFAULT;");
            
            //Fetch results
            $records = $db->fetchAssoc($tables[$id]->getSelect('all', array(
                'querybuilder' => $validData ,
                'limit' => $this->datatableRecordsLimit , 
                'geometryAsWkt' => true
            )));
            
            $result['records'] = array_values($records);
            
            $result['count'] = $countText;
            
            $settings->querybuilder = $validData;
            
        } catch (Exception $e) {
            $settings->querybuilder = NULL;
            $result['error'] = $e->getMessage();
        }
        
        $this->_helper->json($result);
    }
    
    public function getqueriesAction()
    {
        $result['replyCode'] = 200;
        
        $config = Zend_Registry::get('config');
        $queriesTable = new Giscuit_Db_Table_QuerybuilderQueries();

        $rows = $queriesTable->getItems(array(
            'whereUserId' => Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct()
        ));
        
        $result['records'] = $rows->toArray();
        
        $this->_helper->json($result);
    }
    
    public function loadqueryAction()
    {
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        $queriesTable = new Giscuit_Db_Table_QuerybuilderQueries();
        $id = intval($request->getParam('id'));
        
        $row = $queriesTable->getItems(array(
            'whereId' => $id ,
            'whereUserId' => Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct()
        ));
        
        if (count($row) == 0) {
            throw new Giscuit_Exception("Invalid query id ({$id})");
        }

        $row = $row->current();
        
        $result['queryData'] = Zend_Json::decode($row->query);
        
        $this->_helper->json($result);
    }
    
    public function savequeryAction()
    {
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        $id = intval($request->getParam('id'));
        
        $specialCharsFilter = new Giscuit_Filter_HtmlSpecialChars();
        $name = $specialCharsFilter->filter($request->getParam('name'));
        
        try {
            $data = Zend_Json::decode($request->getParam('data'));
            $match = Zend_Json::decode($request->getParam('match'));
        } catch (Exception $e) {
            $result['error'] = $e->getMessage();
            $this->_helper->json($result);
            return;
        }
   
        $save = $request->getParam('save');
        $save = ($request->getParam('save') != 't') ? 'f' : 't';
        
        //Save query
        $queryData = array(
            'id' => $id,
            'data' => $data,
            'match' => $match,
            'save' => $save
        );
        $queryData = Zend_Json::encode($queryData);

        $queriesTable = new Giscuit_Db_Table_QuerybuilderQueries();
        $queriesTable->addItem(array(
            'query' => $queryData,
            'name' => $name
        ));
        
        $this->_helper->json($result);
    }
    
    public function deletequeriesAction()
    {
        $result['replyCode'] = 200;
        
        $queriesTable = new Giscuit_Db_Table_QuerybuilderQueries();
        $request = $this->getRequest();
        $id = intval($request->getParam('ids'));
        
        //Delete queries
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        $where = "id = {$id} AND user_id = {$userId}";
        $queriesTable->delete($where);
        
        $this->_helper->json($result);
    }
    
    public function downloadcreateAction ()
    {
        $config = Zend_Registry::get('config');
        $settings = Zend_Registry::get('settings');
        
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        $id = intval($request->getParam('id'));
        
        $ext = $request->getParam('ext');
        if(! isset($this->formats[$ext])) {
            throw new Giscuit_Exception("Invalid format ({$ext})");
        }

        $connectionString = Giscuit_Controller_Plugin_Init::getDbConnectionString();
        
        $layersTable = new Giscuit_Db_Table_Layers();
        $layers = $layersTable->getItems(array_merge(array(
            'whereId' => $id
        ), $this->_mapOptions));
        
        if (! count($layers)) {    
            throw new Giscuit_Exception("Invalid layer id ({$id})");    
        }
        
        $layer = $layers[$id];
        
        $className = 'Giscuit_Map_Layer_' . $layer['class'];
        if (! @class_exists($className) 
            || ! is_subclass_of($className, 'Giscuit_Map_Layer_Vector')) {
                
            throw new Giscuit_Exception("Invalid layer class ({$className})");
        }
        
        //Tables
        $tableName = 'Giscuit_Db_Table_Layer_' . $layer['class'];
        $tableOptions = array(
            'name' => $layer['table']
        );
        $table = new $tableName($tableOptions);
        $table->setSettings($layer);
        $defaultField = $table->getDefaultField();
        
        $fileName = uniqid(NULL);
        $filePath = GT_TMP_PATH . DIR_SEP . $fileName;
        
        $select = $table->getSelect('all', array(
            'querybuilder' => $settings->querybuilder
        ));
        
        //Quote
        $select = str_replace('"', '\"', (string) $select);
        
        //gml & kml
        if($ext == 'gml' || $ext == 'kml') {
            mkdir($filePath, 0777);
            $ogrFilePath = $filePath . DIR_SEP . $fileName . '.' . $ext;
        } else {
            $ogrFilePath = $filePath;   
        }
        
        if($ext == 'kml') {
            $cmdParams = " -dsco NameField=$defaultField";
        } else if($ext == 'csv') {
            $cmdParams = " -lco SEPARATOR=SEMICOLON";
            
            if (is_subclass_of($className, 'Giscuit_Map_Layer_Point')) {
                $cmdParams .= " -lco GEOMETRY=AS_XYZ";
            } else {
                $cmdParams .= " -lco GEOMETRY=AS_WKT";
            }
        } else {
            $cmdParams = '';
        }

        $cmd = "-s_srs EPSG:{$config->map->srid} -a_srs EPSG:{$config->map->srid} -f \"{$this->formats[$ext]}\" \"{$ogrFilePath}\"" .
            " PG:\"{$connectionString}\"" .
            " -fieldTypeToString Date,Time,DateTime" .
            " -sql \"$select\"" .
            " -lco ENCODING=UTF-8 " .
            $cmdParams;
        
        $cmdResult = Giscuit_Exec::exec('ogr2ogr', $cmd, GT_GDAL_BIN_PATH);
        
        $resFiles = glob($filePath . DIR_SEP . '*');
        
        //Create arhive
        $zip = new ZipArchive();
        $res = $zip->open($filePath . '.zip', ZipArchive::CREATE);
        if (! $res) {
            throw new Giscuit_Exception("Cannot open archive for writing");
        }
        foreach ($resFiles as $resFile) {
            $resParts = pathinfo($resFile);
            $zip->addFile($resFile, $layer['name'] . '.' . $resParts['extension']);
        }
        $zip->close();
        
        $result['file'] = $fileName;
        self::deleteDirectory($filePath);
        
        $this->_helper->json($result);
    }
    
    public function downloadAction ()
    {
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        $file = $this->getRequest()->getParam('file');
        
        $hexValidator = new Zend_Validate_Hex();
        
        if (! $hexValidator->isValid($file)) {
            throw new Giscuit_Exception("Invalid file name ({$file})");
        }
        
        $path = GT_TMP_PATH . DIR_SEP . $file . '.zip';
        if ($fd = fopen($path, "r")) {
            header("Content-type: application/octet-stream");
            header("Content-Disposition: filename=\"giscuit_data.zip\"");
            header("Content-length: " . filesize($path));
            header("Cache-control: private");
            while (! feof($fd)) {
                $buffer = fread($fd, 2048);
                echo $buffer;
            }
        } else {
            throw new Giscuit_Exception("Invalid file ({$file}.zip)");
        }
        
        fclose($fd);
    }
    
    public function getcolumnvaluesAction()
    {
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        
        $layerId = intval($request->getParam('id'));
        $field = $request->getParam('field');
        
        $table = Giscuit_Db_Table_Layers::getLayerTableById($layerId, array(
            'withFields' => true
        ));
        $columns = $table->getPublicFields();

        foreach ($columns as $column) {
            $columnNames[] = $column['name'];
        }
        
        if (! in_array($field, $columnNames)) {
            throw new Giscuit_Exception("Invalid column name ({$field})");
        }
        
        $uniqueValues = $table->getUniqueColumnValues($field, array(
            'limit' => $this->columnValuesLimit
        ));

        $result['records'] = $uniqueValues;
        
        $this->_helper->json($result);
    }
    
    public function clearAction ()
    {
        $settings = Zend_Registry::get('settings');
        
        $result['replyCode'] = 200;
        
        $settings->querybuilder = NULL;
        
        $this->_helper->json($result);
    }
    
    public static function deleteDirectory ($dir)
    {
        if (! file_exists($dir)) {
            return true;
        }
        if (! is_dir($dir)) {
            return unlink($dir);
        }
        
        foreach (scandir($dir) as $item) {
            if ($item == '.' || $item == '..') {
                continue;
            }
            if (! self::deleteDirectory($dir . DIRECTORY_SEPARATOR . $item)) {
                return false;
            }
        }
        
        return rmdir($dir);
    }

}