<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Editing.php 3514 2013-12-03 15:29:38Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Controller_Plugin_Abstract
 */
require_once 'Zend/Controller/Plugin/Abstract.php';

/**
 * Plugin for data editing
 *  
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Controller_Plugin_Editing extends Giscuit_Controller_Plugin_Abstract
{
    /**
     * Main layer opacity during compare
     * 
     * @var integer
     */
    public static $layerOpacity = 25;

    /**
     * List of the javascript file to include
     * 
     * @return array
     */
    public function getJsFilesList ()
    {
        return array(
            'Control/Historydata.js' ,
            'Control/Historymanage.js',
            'Control/Editgeom.js' ,
            'Control/Editattr.js' ,
        );
    }
    
    /**
     * List of the javascript file to include
     * 
     * @return array
     */
    public function getMobileJsFilesList ()
    {
        return array(
            'Util/WebSql.js',
            'Control/Editgeom.js' ,
            'Mobile/Control/Editgeom.js' ,
            'Mobile/Control/Editattr.js' ,
        );
    }

    /**
     * Called on vector layer construction
     * 
     * @param Giscuit_Map_Layer_Vector $layer
     * @return void
     */
    public function vectorLayerConstruct ($layer)
    {
        
    }

    /**
     * Called on adding new database table
     * 
     * @param array $options <ul>Available options:
     * <li>fileName string - Name of the file SQL commands</li>
     * <li>connectionString string - Database connection parameters</li>
     * <li>tableName - Name of the table</li></ul>
     * @return void
     */
    public function adminDatabaseAddItem (array $options = array())
    {
        $config = Zend_Registry::get('config');
        $db = Zend_Registry::get('db');

        $queries = array();
        $tableName = $options['tableName'];
        $schema = "{$config->database->schema}_history";
        
        //Check if data can be appended to history table 
        $geometryTablesTable = new Giscuit_Db_Table_GeometryTables();
        $historyTables = array_keys($geometryTablesTable->getTables(array(
            'schema' => $config->database->schema . '_history'
        )));
        if($options['append'] == true && ! in_array($tableName, $historyTables)) {
            return false;
        }
        
        //Add history record
        $history = new Giscuit_Db_Table_History();
        
        $form = new Giscuit_Form_HistorydataDownload();
        $form->populate(array(
            'comment' => 'Initial import'
        ));
        $hId = $history->addItem($form, array(
            'active' => 't'
        ));

        if(! $options['append']) {
            $queries[] = "ALTER TABLE \"{$schema}\".\"{$tableName}\" DROP COLUMN gid CASCADE";

            $queries[] = "ALTER TABLE \"{$schema}\".\"{$tableName}\" ADD COLUMN hgid serial NOT NULL";
            $queries[] = "ALTER TABLE \"{$schema}\".\"{$tableName}\" ADD CONSTRAINT \"{$tableName}_pkey\" PRIMARY KEY (hgid)";

            $queries[] = "ALTER TABLE \"{$schema}\".\"{$tableName}\" ADD COLUMN gid integer";
            $queries[] = "ALTER TABLE \"{$schema}\".\"{$tableName}\" ADD COLUMN h_id integer";
            $queries[] = "ALTER TABLE \"{$schema}\".\"{$tableName}\" ADD COLUMN hstatus character(1) DEFAULT 'C'::bpchar";

            $queries[] = "UPDATE \"{$schema}\".\"{$tableName}\" SET gid = hgid WHERE gid IS NULL;\n";
        }

        //
        Giscuit_Exec::exec('psql', "-d \"{$options['connectionString']}\" -f \"{$options['fileName']}_history.sql\"", GT_PGSQL_BIN_PATH);

        //
        if($options['append']) {
            $maxGid = $db->fetchRow("SELECT MAX(gid) AS result FROM \"{$config->database->schema}_layers\".\"{$tableName}\"")->result;
            $maxHgid = $db->fetchRow("SELECT MAX(hgid) AS result FROM \"{$config->database->schema}_history\".\"{$tableName}\"")->result;
            $gidOffset = $maxHgid - $maxGid;

            $queries[] = "UPDATE \"{$schema}\".\"{$tableName}\" SET gid = hgid - {$gidOffset} WHERE gid IS NULL;\n";
        }

        //
        $queries[] = "UPDATE \"{$schema}\".\"{$tableName}\" SET h_id = {$hId} WHERE h_id IS NULL;\n";

        //Drop srid constraint
        try {
            $db->query("ALTER TABLE \"{$schema}\".\"{$tableName}\" DROP CONSTRAINT \"enforce_srid_the_geom\"");
        } catch (Exception $e) {
        
        }
        
        //Update table srid
        $db->query("UPDATE public.geometry_columns SET srid = {$config->map->srid} WHERE f_table_schema = '{$schema}' AND f_table_name = '{$tableName}'");
        
        foreach ($queries as $value) {
            $db->query($value);
        }

        unlink("{$options['fileName']}_history.sql");

        return $hId;
    }

    /**
     * Called on database form construction
     * 
     * @param Zend_Form $form 
     * @return void
     */
    public function adminDatabaseForm ($form)
    {
        //Active
        $form->addElement('checkbox', 'history', array(
            'required' => true , 
            'label' => 'enableHistory' , 
            'order' => 9 ,
            'uncheckedValue' => 'f' , 
            'checkedValue' => 't' ,
            'value' => 't'
        ));
    }

    /**
     * Modify the history table to reflect changes
     * 
     * @param integer $gid Row id
     * @param Giscuit_Db_Table_Layer_Abstract $table
     * @param array $options <ul>Available options:
     * <li>hstatus string - Type of the modification C(create) U(update) D(delete)</li></ul>
     * @return array
     */
    public function setItemHistory ($gid, $table, array $options = array())
    {
        if(! ($table instanceof Giscuit_Db_Table_Layer_History)) {
            return;
        }
        
        $historyTable = new Giscuit_Db_Table_History();
        $layersTable = new Giscuit_Db_Table_Layers();
        
        $settings = $table->getSettings();
        
        //Add history header
        $hId = $historyTable->addItem(array(
            'comment' => ''
        ), array(
            'active' => 't'
        ));
        
        //Create history table 
        $historyTableName = 'Giscuit_Db_Table_Layer_' . $settings['class'];
        $historyTable = new $historyTableName(array(
            'name' => $settings['table'] ,
            'withHistory' => true ,
        ));
        $historyTable->setSettings($settings);
        
        $hgid = $historyTable->addFromParent($gid, $table, array(
            'hId' => $hId , 
            'hstatus' => $options['hstatus']
        ));
        
        return array(
            'hgid' => $hgid,
            'h_id' => $hId
        );
    }
    
    public function getItemHistoryGid ($gid, $table, array $options = array())
    {
        if(! ($table instanceof Giscuit_Db_Table_Layer_History)) {
            return;
        }

        $config = Zend_Registry::get('config');
        
        $settings = $table->getSettings();
            
        //Create history table 
        $historyTableName = 'Giscuit_Db_Table_Layer_' . $settings['class'];
        $historyTable = new $historyTableName(array(
            'name' => $settings['table'] ,
            'withHistory' => true ,
        ));
        $historyTable->setSettings($settings);

        //FIXME: Primary key name "gid" is static
        $gidField = $table->getAdapter()->quoteIdentifier("{$table->info('name')}.gid");
        $hidField = $historyTable->getAdapter()->quoteIdentifier($historyTable->info('name') . '.h_id');
            
        $historySelect = $table->select();
        $historySelect->setIntegrityCheck(false);
            
        $historySelect->from($historyTable, array(
            'hgid' => $historyTable->getPrimaryKey()
        )); 
        $historySelect->joinInner(array(
            'history' =>  $config->database->schema . '.history'
        ), "(history.id = $hidField AND history.active = 't')", NULL);

        $historySelect->where("$gidField = ?", $gid);
        
        $historySelect->order('history.activate_date desc');
            
        $historySelect->limit(1);
            
        $historyRow = $historyTable->fetchRow($historySelect);
            
        return $historyRow->hgid;
    }

    /**
     * Update item
     * 
     * @param integer $gid Item id
     * @param Giscuit_Db_Table_Layer_Abstract $table
     * @param string $geometry Geometry represented as WKT
     * @param Zend_Form $data Item data
     * @return void
     */
    public function updateItemForEdit ($gid, $table, $geometry, $data)
    {
        $callbackResult = false;
        
        //Find feature
        $rows = $table->find($gid);
        $row = $rows->current();
        
        $oldRow = clone $row;
        
        //Metadata
        $metadata = $table->info('metadata');
        
        $historyRowGid = $this->getItemHistoryGid($gid, $table);



        if ($data != NULL) {
            $values = $data->getValues();

            foreach ($values as $key => $value) {
                
                //File
                if($data->getElement($key) instanceof Zend_Form_Element_File) {
                    $info = $data->getElement($key)->getFileInfo();
                    $settings = $table->getSettings();
                    
                    //Upload
                    if ($data->getElement($key)->receive() && $data->getElement($key)->getValue() != NULL) {
                        $fileData = array(
                            'file' => $info[$key]['destination'] . DIR_SEP . $info[$key]['name'],
                            'name' => htmlspecialchars($info[$key]['name'], ENT_QUOTES),
                            'layer_id' => $settings['id']
                        );
                        
                        $filesTable = new Giscuit_Db_Table_Files();
                        if (! $table instanceof Giscuit_Db_Table_Layer_History && ! empty($row->$key)) {
                            $filesTable->deleteItem($row->$key);
                        }

                        $id = $filesTable->addItem($fileData);
                    
                        $row->$key = $id;
                        
                    //Id
                    } else {
                        $request = Zend_Controller_Front::getInstance()->getRequest();
                        $fileId = $request->getParam('__fileid_' . $key);
                        
                        if($fileId) {
                            $row->$key = intval($fileId);
                        }
                        
                        if($fileId === '') {
                            $row->$key = NULL;
                        }
                    }
                //Gtmultiselect and Gtmulticheckbox
                } else if($data->getElement($key) instanceof Giscuit_Form_Element_Gtmultiselect ||
                    $data->getElement($key) instanceof Giscuit_Form_Element_Gtmulticheckbox) {
                                        
                    $value = is_array($value) ? implode(', ', $value) : '';

                    if (isset($row->$key)) {
                        $row->$key = $value;
                    }

                //
                } else {
                    //Set default value
                    if($value === "" && ! empty($metadata[$key]['DEFAULT'])) {
                        $value = new Zend_Db_Expr($metadata[$key]['DEFAULT']);    
                    }
                    
                    //If value is an empty string assign a NULL value
                    if ($value === "") {
                        $value = NULL;
                    }
                    
                    if (isset($row->$key)) {
                        $row->$key = $value;
                    }
                }
                
            }

            //Subforms
            $this->subformSetitem($data, $row->gid);

            //
            $callbackResult = $table->updateItemForEditCallback($row, $data, $oldRow);
        }
        
        if($callbackResult != false || $data == NULL) {
            if (! empty($geometry)) {
                $this->updateGeometryForEdit($gid, $table, $geometry);
            }
            
            //Editing history
            if($table instanceof Giscuit_Db_Table_Layer_History) {
                $newItemHistory = $this->setItemHistory($gid, $table, array(
                    'hstatus' => 'U'
                ));
                
                Zend_Registry::get('settings')->editingHistory[] = array(
                    'hstatus' => 'U',
                    'curr' => $newItemHistory['hgid'] ,
                    'prev' => $historyRowGid
                );
            }
        }
    }
    
    /**
     * Add item
     * 
     * @param Giscuit_Db_Table_Layer_Abstract $table
     * @param string $geometry Geometry represented as WKT
     * @param Zend_Form $data Item data
     * @return integer Item id
     */
    public function addItemForEdit ($table, $geometry, $data)
    {
        $values = $data->getValues();
        
        $newRow = $table->createRow();

        foreach ($values as $key => $value) {
            //
            if(! isset($newRow->$key)) {
                continue;
            }

            //File
            if($data->getElement($key) instanceof Zend_Form_Element_File) {
                $info = $data->getElement($key)->getFileInfo();
                $settings = $table->getSettings();
                if ($data->getElement($key)->receive() && $data->getElement($key)->getValue() != NULL) {
                    $fileData = array(
                        'type' => $info[$key]['type'] , 
                        'file' => $info[$key]['destination'] . DIR_SEP . $info[$key]['name'],
                        'name' => htmlspecialchars($info[$key]['name'], ENT_QUOTES),
                        'layer_id' => $settings['id']
                    );

                    $filesTable = new Giscuit_Db_Table_Files();
                    $id = $filesTable->addItem($fileData);
                    
                    $newRow->$key = $id;
                }
            //Gtmultiselect
            } else if($data->getElement($key) instanceof Giscuit_Form_Element_Gtmultiselect || 
                $data->getElement($key) instanceof Giscuit_Form_Element_Gtmulticheckbox) {
                
                $value = is_array($value) ? implode(', ', $value) : '';
                $newRow->$key = $value;
            //
            } else {
                //If value is an empty string assign a NULL value
                if ($value === "") {
                    $value = NULL;
                }
                
                $newRow->$key = $value;
            }
        }

        //
        $pkey = $table->addItemForEditCallback($newRow, $data);

        //Subforms
        $this->subformSetitem($data, $newRow->gid);
        
        //Update the geometry
        if($pkey != false) {
            $this->updateGeometryForEdit($pkey, $table, $geometry);
            
            //Editing history
            if($table instanceof Giscuit_Db_Table_Layer_History) {
                $newItemHistory = $this->setItemHistory($pkey, $table, array(
                    'hstatus' => 'C'
                ));
                
                Zend_Registry::get('settings')->editingHistory[] = array(
                    'hstatus' => 'C',
                    'curr' => $newItemHistory['hgid'] ,
                    'prev' => NULL
                );
            }
        }
        
        return $pkey;
    }
    
    /**
     * Delete item
     * 
     * @param integer $gid Item id
     * @param Giscuit_Db_Table_Layer_Abstract $table
     * @param Zend_Form $form Layer form
     * @return void
     */
    public function deleteItemForEdit ($gid, $table, $form) 
    {
        $gidField = $table->getPrimaryKey(array(
            'quote' => true
        ));

        $callbackResult = $table->deleteItemForEditCallback($gid);
        
        if($callbackResult != false) {
        
            $where = $table->getAdapter()->quoteInto("$gidField = ?", $gid);
            
            $this->setItemHistory($gid, $table, array(
                'hstatus' => 'D'
            ));
            
            if (! $table instanceof Giscuit_Db_Table_Layer_History) {
                $filesTable = new Giscuit_Db_Table_Files();
                $tableCols = $table->info('cols');
                
                foreach($tableCols as $key => $value) {
                    if($form->getElement($value) instanceof Zend_Form_Element_File) {
                        $row = $table->find($gid)->current();
                        if (! empty($row->$value)) {
                            $filesTable->deleteItem($row->$value);
                        }
                    }
                }
            }
            
            $table->delete($where);
            
            //Editing history
            if($table instanceof Giscuit_Db_Table_Layer_History) {
                $historyRowGid = $this->getItemHistoryGid($gid, $table);
                
                Zend_Registry::get('settings')->editingHistory[] = array(
                    'hstatus' => 'D',
                    'curr' => NULL ,
                    'prev' => $historyRowGid
                );
            }
        
        }
    }
    
    /**
     * Update geometry
     * 
     * @param integer $gid Item id
     * @param Giscuit_Db_Table_Layer_Abstract $table
     * @param string $geometry Geometry represented as WKT
     * @param array $options Available options
     * @return void
     */
    public function updateGeometryForEdit ($gid, $table, $geometry, array $options = array())
    {
        $config = Zend_Registry::get('config');
        
        $theGeom = "ST_GeomFromText(:the_geom, {$config->map->srid})";
        
        //Geometry from text
        if (stristr($geometry, 'point')) {

        } else {
            $theGeom = "ST_Multi($theGeom)";
        }
        
        //Parse
        $theGeom = $table->parseGeometryBeforeEdit($theGeom);
        
        $tableName = $table->getAdapter()->quoteIdentifier($table->info('schema') . '.' . $table->info('name'));
        $gidField = $table->getPrimaryKey(array(
            'quote' => true
        ));
        
        $stmt = $table->getAdapter()->prepare("UPDATE $tableName SET the_geom = $theGeom WHERE $gidField = :gid");
        $stmt->bindParam('gid', $gid, PDO::PARAM_INT);
        $stmt->bindParam('the_geom', $geometry, PDO::PARAM_STR);
        
        $result = $stmt->execute();
               
        return $result;
    
    }
    
    /**
     * 
     * @param integer $gid Item id
     * @param Giscuit_Db_Table_Layer_Abstract $table
     * @return Zend_Db_Table_Rowset 
     */
    public function getItemForEdit ($gid, $table, $data, $version = NULL)
    {
        
        $select = $table->select();
        $select->setIntegrityCheck(false);
        
        $theGeomField = $table->getAdapter()->quoteIdentifier($table->info('name') . '.the_geom');
        
        $fields = $table->info('cols');
        
        //Remove the_geom field
        unset($fields[array_search('the_geom', $fields)]);
        
        $fields['the_geom'] = new Zend_Db_Expr("st_astext($theGeomField)");
        
        $select->from($table, $fields);
        
        if(! empty($version)) {
            $gidField = $table->getAdapter()->quoteIdentifier($table->info('name') . '.gid');
            
            $hidField = $table->getAdapter()->quoteIdentifier($table->info('name') . '.h_id');
            $select->where("$hidField = ?", $version);
            
        } else {
            $gidField = $table->getPrimaryKey(array(
                'quote' => true
            ));
        }
        
        $select->where("$gidField = ?", $gid);
        
        //Result
        $result = $table->fetchRow($select);
        
        //
        if(! empty($version) && count($result) == 0) {
            throw new Giscuit_Db_Table_Exception('Invalid version (' . $version . ')');
        }
        
        //
        foreach ($result as $key => $value) {
        
            //Gtmultiselect
            if($data->getElement($key) instanceof Giscuit_Form_Element_Gtmultiselect 
                || $data->getElement($key) instanceof Giscuit_Form_Element_Gtmulticheckbox) {
                
                $result->$key = explode(', ', $value);
            }
        }
        
        return $table->getItemForEditCallback($result, $data);
    }
    
    /**
     * Get parameters
     * 
     * @return array
     */
    public function getParams() 
    {
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        
        return array(
            'historymanageIsAllowed' => $isAllowed->direct('tool_historymanage_controller') ? true : false,
            'historydataActivatesubmitIsAllowed' => $isAllowed->direct('tool_historydata_controller', 'activatesubmit') ? true : false,
        );    
    }

    /**
     * Insert subform items
     *
     * @return void
     */
    public function subformSetitem($form, $parentId)
    {
        //
        $db = Zend_Registry::get('db');
        $config = Zend_Registry::get('config');

        //
        $groups = $form->getDisplayGroups();
        $params = Zend_Controller_Front::getInstance()->getRequest()->getParams();

        //FIXME: Disable for mobile
        if($params['module'] == 'mobile') {
            return;
        }

        //
        foreach($groups as $group) {
            $attrs = $group->getAttribs();

            if(isset($attrs['subform']) && $attrs['subform']) {
                $data = array();
                $els = $group->getElements();
                $attrs = $group->getAttribs();

                //Get table and schema names
                $dbParams = $attrs['database'];
                $table = $dbParams['table'];
                $schema = isset($dbParams['schema']) ? $options['database']['schema'] : $config->database->schema . '_layers';
                //
                $i=0;
                foreach($els as $elName => $el) {
                    //Skip default values
                    $ddClass = $el->getDecorator('HtmlTag')->getOption('class');
                    if(stristr($ddClass, 'gtSubformDefault')) {
                        continue;
                    }

                    //Get field name and index
                    $fieldName = $this->_splitFieldName($elName);

                    foreach($params as $paramKey => $param) {
                        $paramName = $this->_splitFieldName($paramKey);

                        if(! $paramName || $paramName[1] != $fieldName[1] || ! is_numeric($fieldName[0])) {
                            continue;
                        }

                        $value = $params[$paramName[1] . '_' . $paramName[0]];

                        //Gtmultiselect & Gtmulticheckbox
                        if($el instanceof Giscuit_Form_Element_Gtmultiselect ||
                            $el instanceof Giscuit_Form_Element_Gtmulticheckbox) {

                            $value = is_array($value) ? implode(', ', $value) : '';
                        }

                        //If value is an empty string assign a NULL value
                        if ($value === "") {
                            $value = NULL;
                        }

                        //
                        $data[$paramName[0]][$fieldName[1]] = $value;
                        $data[$paramName[0]]['parent_id'] = $parentId;
                    }
                }

                //Insert
                $db->delete($schema . '.' . $table, $db->quoteInto('parent_id = ?', $parentId));
                foreach($data as $row) {
                    $db->insert($schema . '.' . $table, $row);
                }

            }
        }
    }

    protected function _splitFieldName($fieldName) {
        $fieldName = explode('_', $fieldName);

        if(count($fieldName) < 2) {
            return false;
        }

        $fieldIdx = $fieldName[count($fieldName) - 1];
        unset($fieldName[count($fieldName) - 1]);
        $fieldName = implode('_', $fieldName);

        return array(
            $fieldIdx,
            $fieldName
        );
    }
}
