<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Quickaccess.php 2897 2012-08-02 11:41:08Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Class for quickaccess table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Layer_Quickaccess extends Giscuit_Db_Table_Layer_Abstract
{

    const INSERT = 1;
    const UPDATE = 2;

    /**
     * Get the current select object of table
     *  
     * @param string $dump - What data should be requested from the layer (gid_geometry, gid_geometry_label, all, all_table, count)
     * @param array $options Available options:
     * <ul><li>geometryAsWkt bool - Return geometry in WKT format</li>
     * <li>geometryAsGeoJSON bool - Return geometry in GeoJSON format</li>
     * <li>withIntersection string - Table name and geometry field (table.the_geom) that will be intersected with the current geometry. Returns the product of the intersection as the geometry.</li>
     * <li>whereIntersection string - Fetch rows that intersect specified table name and geometry (table.the_geom)</li>
     * <li>whereStm string - Fetch rows that match the specified statement</li>
     * <li>fields array - An array [[name, visibility, expr]] of additional field to fetch</li>
     * <li>form Zend_Form - Edit attributes form object of this layer</li>
     * <li>limit integer - Number of rows to fecth</li></ul>
     * @return Zend_Db_Select
     */
    public function getSelect ($dump = 'gid_geometry_label', array $options = array())
    {
        $select = parent::getSelect($dump, $options);
        if (! $select) {
            return;
        }
        
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        if (isset($options['forMenu'])) {
            if ($userId) {
                $select->where("(quickaccess.published = 't' OR quickaccess.user_id = ?)", $userId);
            } else {
                $select->where("(quickaccess.published = 't')");
            }
            $select->order("quickaccess.published DESC");
        } else {
            $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
            
            if ($isAllowed->direct('admin')) {
                $select->where("({$this->_name}.published = 't' OR {$this->_name}.user_id = ?)", $userId);
            } else {
                $select->where($this->_name . '.user_id = ?', $userId);
            }
        }

        return $select;
    }
    
    /**
     * Add item
     * 
     * @param Zend_Db_Table_Row $row New row
     * @param Zend_Form $form Form object populated with request data
     * @return integer $pkey Row id
     */
    public function addItemForEditCallback($row, $form = NULL)
    {
        $languages = array_keys(Zend_Registry::get('config')->language->toArray());
        $langValues = array();
        foreach ($languages as $lang) {
            $langValues[$lang] = $form->getElement("name_{$lang}")->getValue();
        }
        
        $this->manageMultilangText(array('name' => $langValues), $row);
        $row->user_id = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        $pkey = $row->save();
        
        return $pkey;
    }
    
    /**
     * Update item
     * 
     * @param Zend_Db_Table_Row $row Updated row
     * @param Zend_Form $form Form object populated with request data
     * @return bool Update/don't update item
     */
    public function updateItemForEditCallback($row, $form = NULL, $oldRow)
    {
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        if(! $isAllowed->direct('admin') && $userId != $row->user_id) {
            return false;
        }
        
        $languages = array_keys(Zend_Registry::get('config')->language->toArray());
        $langValues = array();
        foreach ($languages as $lang) {
            $langValues[$lang] = $form->getElement("name_{$lang}")->getValue();
        }
        
        $this->manageMultilangText(array('name' => $langValues), $row, self::UPDATE);
        
        $row->save();
        
        return true;
    }
    
    /**
     * Get item
     * 
     * @param Zend_Db_Table_Row $row Updated row
     * @param Zend_Form $form Form object populated with request data
     * @return bool Update/don't update item
     */
    public function getItemForEditCallback($row, $form = NULL)
    {
        $languages = array_keys(Zend_Registry::get('config')->language->toArray());
        $textTable = new Giscuit_Db_Table_Text();
        
        $textRow = $textTable->find($row->name_id);
        $textRow = $textRow->current();
        
        foreach ($languages as $lang) {
            $form->getElement("name_{$lang}")->setValue($textRow->$lang);
        }
        
        return $row;
    }
    
    public function deleteItemForEditCallback($gid)
    {
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        $row = $this->find($gid)->current();
        
        if(! $isAllowed->direct('admin') && $userId != $row->user_id) {
            return false;
        }
        
        return true;
    }
    
    public function manageMultilangText (array $arr, $row, $action = self::INSERT)
    {
        $languages = array_keys(Zend_Registry::get('config')->language->toArray());
        $textTable = new Giscuit_Db_Table_Text();
        
        foreach ($arr as $fieldKey => $field) {
            
            $fieldName = $fieldKey . '_id';
            
            if ($action == self::INSERT) {
                
                //Create new row
                $textRow = $textTable->createRow();
            
            } elseif ($action == self::UPDATE) {
                
                //Find row
                $textRow = $textTable->find($row->$fieldName);
                $textRow = $textRow->current();
            
            }
            
            //Update text fields
            foreach ($field as $langKey => $text) {
                if (in_array($langKey, $languages)) {
                    $textRow->$langKey = $text;
                }
            }
            
            if ($action == self::INSERT) {
                
                $fieldId = $textRow->save();
                $row->$fieldName = $fieldId;
            
            } elseif ($action == self::UPDATE) {
                $textRow->save();
            }
        }
    
    }
}
