<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Rastercut.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Db_Table_Abstract
 */
require_once 'Zend/Db/Table/Abstract.php';

/**
 * Rastercut table class 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Layer_Rastercut extends Giscuit_Db_Table_Layer_Abstract
{
    
    /**
     * Is called after getSelect to apply custom logic
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options
     * @return void
     */
    protected function _selectCallback ($select, array $options = array())
    {
        parent::_selectCallback($select, $options);
        
        $settings = Zend_Registry::get('settings');
        
        //Interval
        if (! empty($settings->rastercutDateBegin)) {
            $select->where($this->_name . '.date > ?', new Zend_Db_Expr("to_timestamp('{$settings->rastercutDateBegin}', '" . GT_DATETIME_STYLE_DB . "')"));
        } else {
            $select->where($this->_name . '.date > ?', new Zend_Db_Expr('current_date::timestamp'));
        }
        
        if (! empty($settings->rastercutDateEnd)) {
            $select->where($this->_name . '.date < ?', new Zend_Db_Expr("to_timestamp('{$settings->rastercutDateEnd}', '" . GT_DATETIME_STYLE_DB . "') + interval '1 day' - interval '1 second'"));
        }
        
        $select->order($this->_name . '.area desc');
    
    }

    /**
     * Add new item and return it's id
     * 
     * @param Zend_Form $data
     * @return integer
     */
    public function addItem ($data)
    {
        $config = Zend_Registry::get('config');
        
        //User id
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        $geometrySql = "st_multi(st_geomfromtext('" . $data->getValue('geometry') . "', {$config->map->srid}))"; 
        
        $newRow = $this->createRow();
        
        $newRow->comment = $data->getValue('comment');
        $newRow->the_geom = new Zend_Db_Expr($geometrySql);
        $newRow->user_id = $userId;
        $newRow->scale = $data->getValue('scale');
        
        $newRow->area = new Zend_Db_Expr($this->getArea($geometrySql));
        
        return $newRow->save();
    
    }

    /**
     * Get items by date
     * 
     * @return Zend_Db_Table_Rowset
     */
    public function getByDate ()
    {
        $select = $this->getSelect('all', array(
            'geometryAsWkt' => true            
        ));
        
        //Order
        $select->order($this->_name . '.area desc');
        
        //Result
        $result = $this->fetchAll($select);
        
        foreach ($result as $row) {
            $row->comment = htmlspecialchars($row->comment, ENT_QUOTES);
            $row->area = floatval($row->area);
            $row->scale = intval($row->scale);
        }
        
        return $result;
    }
    
    /**
     * Called after getPublicFields to apply custom logic
     * 
     * @param array $fieldsArr
     * @param array $options Available options
     * @return array
     */
    protected function _publicFieldsCallback (array $fieldsArr, array $options = array())
    {
        $fieldsArr = parent::_publicFieldsCallback($fieldsArr);
        $fieldsArr['area']['expr'] = new Zend_Db_Expr("round(" . $this->getArea($this->_name . '.the_geom') . "::numeric, 3)");
        $fieldsArr['date']['expr'] = new Zend_Db_Expr("to_char({$this->_name}.date, '" . GT_DATETIME_STYLE_DB . "')");
         
        return $fieldsArr;
    }
    
    /**
     * Called after getSelect to join tables to current table
     * 
     * @param Zend_Db_Select $select
     * @return void
     */
    protected function _joinTablesCallback ($select)
    {
        parent::_joinTablesCallback($select);
                
        $select->joinLeft(
            $this->_mainSchema . '.' . 'users' , 
            $this->_name . '.user_id = users.id' ,
            NULL
        );
        
    }
    
    /**
     * Called after _querybuilder
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options
     * @return void
     */
    protected function _querybuilderCallback ($select, array $options = array())
    {
        $select->reset('where');    
        $select->reset('order');
    }

}

