<?php
/**
 * Giscuit
 *
 * @version    $Id: DatabaseColumn.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 *
 */

/**
 * @see Zend_Form
 */
require_once 'Zend/Form.php';

/**
 * Alter database table colum form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 *
 */
class Giscuit_Form_Admin_DatabaseColumn extends Giscuit_Form_Abstract
{
    
    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        //Column name
        $this->addElement('text', 'name', array(
            'filters' => array(
                'StringToLower'
            ) , 
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    3
                ) ,
                new Zend_Validate_Regex('/^[^0-9][a-zA-Z0-9\_]+$/')
            ) , 
            'required' => true , 
            'label' => 'name'
        ));
        
        //Column type
        $this->addElement('select', 'type', array(
            'filters' => array(
                'StringToLower'
            ) , 
            'multiOptions' => $this->getColumnTypes() , 
            'required' => true , 
            'label' => 'type'
        ));
        
        //Type length
        $this->addElement('text', 'length', array(
            'required' => false , 
            'label' => 'length'
        ));
        
        //Not null
        $this->addElement('checkbox', 'notnull', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'notnull' , 
            'checkedValue' => 't' , 
            'uncheckedValue' => 'f'
        ));
        
        //Default value
        $this->addElement('text', 'default', array(
            'filters' => array(
                'StringTrim'
            ) , 
            'required' => false , 
            'label' => 'default'
        ));
        
        //Edit existing layers
        $this->addElement('checkbox', 'modifyLayers', array(
            'validators' => $this->checkboxValidator , 
            'required' => true , 
            'label' => 'editLayers' , 
            'checkedValue' => 't' , 
            'uncheckedValue' => 'f' ,
            'checked' => true
        ));
        
        //Old column name
        $this->addElement('hidden', 'oldName', array(
            'filters' => array(
                'StringToLower'
            ) , 
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    3
                ) ,
                /* 
                new Zend_Validate_Regex('/^[^0-9][a-zA-Z0-9]+$/')
                */
            ) , 
            'required' => false
        ));
    
    }

    public function getColumnTypes ()
    {
        $typesArr = array(
            "smallint" => "smallint" , 
            "integer" => "integer", 
            "bigint" => "bigint", 
            "numeric" => "numeric", 
            "real" => "real", 
            "serial" => "serial", 
            "double precision" => "double precision", 
            "character varying" => "character varying", 
            "character" => "character", 
            "text" => "text", 
            "bytea" => "bytea", 
            "timestamp without time zone" => "timestamp without time zone", 
            "timestamp with time zone" => "timestamp with time zone", 
            "interval" => "interval", 
            "date" => "date", 
            "time without time zone" => "time without time zone", 
            "time with time zone" => "time with time zone", 
            "boolean" => "boolean"
        );
        
        return $typesArr;
    }

}

?>