<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: PrivatelayersSettings.php 2943 2012-08-17 07:00:28Z vlad.bruma $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Form_Abstract
 */
require_once 'Giscuit/Form/Abstract.php';

/**
 * Layer settings label form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_PrivatelayersSettings extends Giscuit_Form_Abstract
{
    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        $this->setMethod('post');
        $this->addAttribs(array(
            'id' => 'gtPrivatelayersSettingsForm'
        ));
        
        //View
        $this->addElement('checkbox', 'view', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'view' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Select
        $this->addElement('checkbox', 'select', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'information' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Label
        $this->addElement('checkbox', 'label', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'label' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Edit
        $this->addElement('checkbox', 'edit', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'edit' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Size
        $this->addElement('text', 'label_size', array(
            'validators' => array(
                'Digits' , 
                array(
                    'Between' , 
                    false , 
                    array(
                        4 , 
                        32
                    )
                )
            ) , 
            'required' => false , 
            'label' => 'size'
        ));
        
        //Color
        $this->addElement('hidden', 'label_color', array(
            'validators' => array(
                'Color'
            ) , 
            'required' => true , 
            'label' => 'color' , 
            'decorators' => array(
                'ViewHelper' , 
                'Errors' , 
                array(
                    'data' => 'HtmlTag' , 
                    array(
                        'tag' => 'div' , 
                        'id' => 'gtPrivatelayersLabelColorContainer'
                    )
                ) , 
                array(
                    'Label' , 
                    array(
                        'tag' => 'dt'
                    )
                )
            ) , 
            'attribs' => array(
                'id' => 'gtPrivatelayersLabelColorValue'
            )
        ));
        
        //Fields
        $this->addElement('multiCheckbox', 'label_fields', array(
            'required' => false , 
            'label' => 'fields' , 
        ));
                
        //Submit
        $this->addElement('button', 'submit_button', array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'save' , 
            'id' => 'gtPrivatelayersLabelSubmit' , 
            'decorators' => array(
                'ViewHelper'
            )
        ));
        
        //Back
        $this->addElement('button', 'back_button', array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'back' , 
            'id' => 'gtPrivatelayersLabelBack' , 
            'decorators' => array(
                'ViewHelper'
            )
        ));
        
    }

    /**
     * Populate form
     *
     * Proxies to {@link setDefaults()}
     *
     * @param  array $values
     * @return Zend_Form
     */
    public function populate (array $values)
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $config = Zend_Registry::get('config');
        
        if(empty($values['id'])) {
            throw new Exception('Empty id');
        }
        
        $values['id'] = intval($values['id']);
        
        $mapOptions = array(
            'orderDirection' => 'asc' , 
            'whereActive' => 't' , 
            'withLayerSettings' => true , 
            'withFields' => true , 
            'withAcl' => true , 
            'id' => $values['id']
        );
        $table = Giscuit_Db_Table_Layers::getLayerTableById($values['id'], $mapOptions);
        
        if($table) {
            $settings = $table->getSettings();
            
            //Label fields
            $fields = $table->getPublicFields();
            $fieldsArr = array();
            $maxLength = (count($fields) > $config->maxFieldsAmount) ? $config->maxFieldsAmount : count($fields);
            $i = 0;
            foreach ($fields as $field) {
                $fieldsArr[$field['name']] = $translate->_($field['name']);
                $i++;
                if ($i >= $maxLength) {
                    break;
                }
            }
            $labelFields = $this->getElement('label_fields');
            $labelFields->setMultiOptions($fieldsArr);
            $labelFields->setValidators(array(
                array(
                    'InArray' , 
                    false , 
                    array(
                        array_keys($fieldsArr)
                    )
                )
            ));
        }
        
        //Populate from database is no values are supplied
        if(count($values) == 1) {
            $values = $settings;
        }
        
        $result = $this->setDefaults($values);
        
        return $result;
    }

}