<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Raster.php 2844 2012-07-09 08:54:35Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Map
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * @see Giscuit_Map_Layer_Abstract
 */
require_once 'Giscuit/Map/Layer/Abstract.php';

/**
 * Class for abstract mapserver raster layer object
 *
 * @category   Giscuit
 * @package    Giscuit_Map
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
abstract class Giscuit_Map_Layer_Raster extends Giscuit_Map_Layer_Abstract
{

    /**
     * Class constructor
     *
     * @param mixed $map Map object
     * @param array $settings Settings
     * @return void
     */
    public function __construct ($map, $settings)
    {
        parent::__construct($map, $settings);
        
        $config = Zend_Registry::get('config');
        
        /*
        if (! empty($this->_settings['data'])) {
            if(realpath($this->_settings['data']) === false) {
                $this->set('data', GT_RASTER_PATH . DIR_SEP . $this->_settings['data']);
            } else {
                $this->set('data', $this->_settings['data']);
            }
        }
        */
        
        if (! empty($this->_settings['data'])) {
            $this->set('data', $this->_settings['data']);
        }
        
        if (! empty($this->_settings['tileitem'])) {
            $this->set('tileitem', $this->_settings['tileitem']);
        }
        
        //FIXME:  
        if (! empty($this->_settings['tileindex'])) {
            $this->set('tileindex', $this->_settings['tileindex']);
        }
        
        if(! empty($this->_settings['srid']) && $this->_settings['srid'] != $config->map->srid) {
            $this->transform = MS_TRUE;
            $this->setProjection('init=epsg:' . $this->_settings['srid']);
        } else {
            $this->setProjection('init=epsg:' . $config->map->srid);
        }
        
        //Offsite
        if (! empty($this->_settings['offsite'])) {
            $offsite = Giscuit_Map_Layer_Vector::hexToRgb($this->_settings['offsite']);
            $this->offsite->setrgb($offsite[0], $offsite[1], $offsite[2]);
        }
        
    }
    
    /**
     * Get data by point coordinates
     * 
     * @param float $lon
     * @param float $lat
     * @return array
     */
    public function getByPoint($lon, $lat)
    {
        $point = ms_newPointObj();
        $point->setXY($lon, $lat);
        
        $this->set("template", GT_CONFIG_PATH . DIR_SEP . 'blank.html');
        
        $this->open();
        
        $query = @$this->queryByPoint($point, MS_SINGLE, 0);
        
        if($query == MS_FAILURE || ! $this->getNumResults()) {
            return array();
        }
                        
        $result = $this->getResult(0);
        
        //Get shape
        $msVersion = ms_GetVersionInt();
        if($msVersion >= 60000) {
            $shape = $this->getShape($result);
        } else {
            $shape = $this->getShape($result->shapeindex, $result->tileindex);    
        }
        
        $bands = explode(',', $shape->values['value_list']);
        
        $this->close();
        
        return array(
            'values' => $this->_getByPointResultCallback($bands) , 
            'x' => $shape->values['x'] ,
            'y' => $shape->values['y']
        );
    }
    
    /**
     * Called on parsing data of the getByPoint method
     * 
     * @param array $bands
     * @return array
     */
    protected function _getByPointResultCallback($bands)
    {
        return $bands;
    }
}
