<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: ConfirmPassword.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Validate
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Validator for confirming users password
 * 
 * @category   Giscuit
 * @package    Giscuit_Validate
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Validate_ConfirmPassword extends Zend_Validate_Abstract
{

    const PASSWORDS_NOT_MATCH = 'passwordsNotMatch';

    /**
     * Validation failure message template definitions
     *
     * @var array
     */
    protected $_messageTemplates = array(
        self::PASSWORDS_NOT_MATCH => 'Passwords do not match'
    );

    /**
     * Field to compare with
     * 
     * @var Zend_Validate_Abstract
     */
    protected $_field;

    /**
     * Sets validator options
     *
     * @param Zend_Validate_Abstract $field
     * @return void
     */
    public function __construct ($field = null)
    {
        $this->setField($field);
    }

    /**
     * Get the field
     *
     * @return Zend_Validate_Abstract
     */
    public function getField ()
    {
        return $this->_field;
    }

    /**
     * Set the field
     * 
     * @param mixed $field
     * @return Zend_Validate_Abstract
     */
    public function setField ($field)
    {
        $this->_field = $field;
        return $this;
    }

    /**
     * Defined by Zend_Validate_Interface
     *
     * Returns true if and only if $value is equal to $context
     *
     * @param string $value 
     * @param string $context
     * @return boolean
     */
    public function isValid ($value, $context = null)
    {
        $value = (string) $value;
        $this->_setValue($value);
        
        if (is_array($context)) {
            if (isset($context[$this->_field]) && ($value == $context[$this->_field])) {
                return true;
            }
        } elseif (is_string($context) && ($value == $context)) {
            return true;
        }
        
        $this->_error(self::PASSWORDS_NOT_MATCH);
        return false;
    }
}