<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Geometry.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Validate
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * WKT geometry validator
 * 
 * @category   Giscuit
 * @package    Giscuit_Validate
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Validate_Geometry extends Zend_Validate_Abstract
{

    const INVALID_GEOMETRY = 'invalidGeometry';

    /**
     * Validation failure message template definitions
     *
     * @var array
     */
    protected $_messageTemplates = array(
        self::INVALID_GEOMETRY => "Invalid geometry"
    );

    /**
     * Defined by Zend_Validate_Interface
     * 
     * Returns true if and only if $value is a valid WKT geometry
     * 
     * @param string $value
     * @return boolean
     */
    public function isValid ($value)
    {
        $config = Zend_Registry::get('config');
        $db = Zend_Registry::get('db');
                
        $value = htmlspecialchars($value, ENT_QUOTES);
        
        $theGeom = "st_geomfromtext(:the_geom, {$config->map->srid})";
                
        $stmt = $db->prepare("SELECT ST_IsValidReason($theGeom) as result");
        $stmt->bindParam('the_geom', $value, PDO::PARAM_STR);
        
        try {
            $stmt->execute();
            $result = $stmt->fetch();
            
            if ($result->result == 'Valid Geometry') {
                return true;
            } else {
                $this->_error($result->result);
                return false;
            }
        } catch (Zend_Db_Exception $e) {
            $this->_error(self::INVALID_GEOMETRY);
            return false;
        }
    
    }
}