<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: ConfirmPassword.php 2182 2012-01-20 12:01:13Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Validate
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Validator for confirming users password
 * 
 * @category   Giscuit
 * @package    Giscuit_Validate
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Validate_TableExists extends Zend_Validate_Abstract
{

    const TABLE_NOT_EXISTS = 'tableDoesNotExist';

    /**
     * Validation failure message template definitions
     *
     * @var array
     */
    protected $_messageTemplates = array(
        self::TABLE_NOT_EXISTS => 'Table does not exist'
    );
    
    /**
     * Options
     * 
     * @var array Table schema and name
     */
    protected $_tableOptions;
    
    /**
     * Sets validator options
     *
     * @param  array|Zend_Config $haystack
     * @return void
     */
    public function __construct($options)
    {
        $this->setTableOptions($options);
    }
    
    /**
     * Get table options
     *
     * @return array
     */
    public function getTableOptions ()
    {
        return $this->_tableOptions;
    }

    /**
     * Set table options
     * 
     * @param mixed $field
     * @return Zend_Validate_Abstract
     */
    public function setTableOptions ($tableOptions)
    {
        $this->_tableOptions = $tableOptions;
        return $this;
    }

    /**
     * Defined by Zend_Validate_Interface
     *
     * Returns true if and only table defined by options exists
     *
     * @param string $value 
     * @param string $context
     * @return boolean
     */
    public function isValid ($value)
    {
        $db = Zend_Registry::get('db');
        
        $value = (string) $value;
        
        //
        if($value == 'f') {
            return true;
        }

        //        
        $tableOptions = $this->getTableOptions();
        $row = $db->fetchRow("SELECT * FROM pg_tables WHERE schemaname='{$tableOptions[0]}_routing' AND tablename='{$tableOptions[1]}'");
        if ($row) {
            return true;
        }
        
        $this->_error(self::TABLE_NOT_EXISTS);
        return false;
    }
}