<?php

//Patch information
$currentVersion = '1.4.2';
$updateVersion = '1.4.3';
$downloadsUrl = 'http://downloads.giscuit.com';

//Defines
require_once '..' . DIRECTORY_SEPARATOR . 'Defines.php';
if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
    define('GT_MS_PATH', realpath(GT_APP_PATH . DIR_SEP . '..' . DIR_SEP . '..' . DIR_SEP . '..'));    
    } else {
    define('GT_MS_PATH', NULL);
}

//Errors
error_reporting(E_ALL | E_STRICT);
ini_set('display_startup_errors', 1);
ini_set('display_errors', 1);
ini_set('log_errors', 0);

//Include_path
$paths = array(
    realpath(GT_LIBRARY_PATH) , 
    get_include_path()
);
set_include_path(implode(PATH_SEPARATOR, $paths));

//Loader
require_once 'Zend/Loader/Autoloader.php';
$loader = Zend_Loader_Autoloader::getInstance();
$loader->registerNamespace('Giscuit_');

//Translate
$translate = new Zend_Translate('tmx', GT_LANGUAGES_PATH . DIR_SEP . 'admin.en.tmx', 'en');
Zend_Registry::set('Zend_Translate', $translate);

//Config
$config = new Zend_Config_Xml(GT_CONFIG_PATH . DIR_SEP . 'config.xml', null, array(
    'skipExtends' => true , 
    'allowModifications' => true
));

//
$view = new Zend_View();
$view->type = 'patch';
$view->httpPath = $config->production->httpPath;
$view->licenseType = $config->production->licenses->type;


//
$view->params = '';

//Install 
if (isset($_GET['install']) && $_GET['install'] == 't') {
	$view->install = 't';
	
	if ($config->production->version == $updateVersion) {
    	$view->bd = "System was already updated to this version";
    	
	} else if ($config->production->version != $currentVersion) {
    	$view->bd = "This patch is for version {$currentVersion}";
	    
	} else {
	    //Initialize database
	    Giscuit_Controller_Plugin_Init::parseDbParams($config->production->database);
	    $db = Zend_Db::factory($config->production->database);
	    $db->getConnection();
	    Zend_Db_Table_Abstract::setDefaultAdapter($db);
	    
	    $db->beginTransaction();
	    try {
	    	//Patch process BEGIN

	        //Update languages
	        updateLanguages(array(
	            'en' => array(
                    'recommendMobileChrome' => 'Use Chrome for Mobile for the best experience',
                    'zoomToLayer' => 'Zoom to layer',
                    'netcheck' => 'Connection check interval',
                    'visibleLayersLimitReached' => 'Visible layers limit reached, performance is degraded, please disable some of the visible layers.'
                ),
	            'ro' => array(
                    'recommendMobileChrome' => 'Utilizați Chrome for Mobile pentru cea mai bună experiență',
                    'zoomToLayer' => 'Măriți la strat',
                    'netcheck' => 'Interval de verificare a conexiunii',
                    'visibleLayersLimitReached' => 'Ați ajuns la limita straturilor vizibile, performanță poate fi degradată, vă rugăm să dezactivați unele dintre straturile vizibile.'

	            ),
	            'ru' => array(
                    'recommendMobileChrome' => 'Используйте Chrome for Mobile для повышения удобства работы',
                    'zoomToLayer' => 'Приблизить к слою',
                    'netcheck' => 'Интервал проверки подключения',
                    'visibleLayersLimitReached' => 'Предел видимых слоев достигнут, производительность может ухудшится, пожалуйста отключите некоторые из видимых слоев.'
	            )
	        ));
	        
	        //Update tables
	        $defaultSchema = $config->production->database->schema;
	        
	        $patchSql = array(
                "ALTER TABLE \"{$defaultSchema}\".users ADD COLUMN netcheck integer DEFAULT 5"
	        );
	        
	        foreach ($patchSql as $value) {
	            $db->query($value);
	        }
	        $db->commit();
	        
	        //Update config
	    	
	        //Patch process END
	        $config->production->version = $updateVersion;
	        
	        //
	        require_once GT_MODULES_PATH . DIR_SEP . 'admin' . DIR_SEP . 'controllers' . DIR_SEP . 'GenconfController.php';
	        Admin_GenconfController::escapeConfig($config->production);
	        
	        //
	        $writer = new Zend_Config_Writer_Xml(array(
	            'config' => $config , 
	            'filename' => GT_CONFIG_PATH . DIR_SEP . 'config.xml'
	        ));
	        $writer->write();
	        
	        //License
	        if(isset($_POST['agreement_text']) && $_POST['agreement_text']) {
	            file_put_contents(GT_APP_PATH . DIR_SEP . "LICENSE.txt", $_POST['agreement_text']);
	        }
	        
	        //Clear cache
	        $frontendOptions = array(
	            'lifetime' => 3600 , 
	            'automatic_serialization' => true
	        );
	        if(extension_loaded('apc')) {
	            $backendOptions = array(
	            );
	            $backend = 'Apc';            
	        } else {
	            $backendOptions = array(
	                'cache_dir' => GT_TMP_PATH
	            );
	            $backend = 'File';
	        }
	        $cache = Zend_Cache::factory('Core', $backend, $frontendOptions, $backendOptions);
	        $cache->clean(Zend_Cache::CLEANING_MODE_ALL);
	        
	        //Update cache
	        file_put_contents(GT_TMP_PATH . DIR_SEP . 'offline_timestamp', time());
	    
	        $view->bd = "Giscuit was successfully patched to version " . $updateVersion;
	    
	    } catch (Exception $e) {
	        $db->rollBack();
	        $view->bd = $e->getMessage();
		}
	}
} else {
	$view->install = 'f';
	
    $contentView = new Zend_View();
    $contentView->type = 'patch';
    $contentView->addScriptPath(GT_MODULES_PATH . DIR_SEP . 'default' . DIR_SEP . 
        'views' . DIR_SEP . 'scripts' . DIR_SEP . 'generic');
    
    $view->bd = $contentView->render('installLicenseAgreement' . GT_SCRIPT_EXT);
    
}

$view->addScriptPath(GT_LAYOUTS_PATH);
echo $view->render('install' . GT_SCRIPT_EXT);

function updateLanguages(array $messages = array()) {
    foreach ($messages as $languageKey => $languageValue) {
        
        $languageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx';
        if(! file_exists($languageFile)) {
            continue;       
        }
        
        $translate = new Zend_Translate('tmx', $languageFile, $languageKey);
    
        $newMessages = array_merge($translate->getMessages(), $languageValue);
        $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx($languageKey, $newMessages);
        
        file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx', $newTmx);
    }    
}