/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Editattr.js 3544 2014-04-24 16:08:30Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Control.Editattr
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Editattr = OpenLayers.Class(Giscuit.Control, {

    feature: null,

    panel: null,

    submitButton: null,
    
    cancelButton: null,
    
    versionsSelect: null,
    
    dateInstances: null,
    
    initialize: function (feature, version) {
        if(feature != null) {
            this.feature = feature;
        }
        
        var getCallback = {
            success: this.getSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };
        
        if(this.panel == null) {
            gt.accord.addPanel({
                label: gt.translate._('editAttributes') ,
                content: this.getLoadingImage()
            });
            this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
            this.addHelpButton('http://docs.giscuit.com/En/tutorial_refguide.user.desktop.edit.pkg.html#online');
        } else {
            gt.accord.setBody(this.getPanelIndex(), this.getLoadingImage());
        }   
        gt.accord.openPanel(this.getPanelIndex());

        //
        var idStm = '';
        if(this.feature.attributes.gid != null) {
            idStm = "id=" + this.feature.attributes.gid;
        }
        var geometryStm = '';
        if(this.feature.geometry != null) {
            geometryStm = "geometry=" + encodeURIComponent(this.feature.geometry.toString());
        }
        var versionStm = 'version=' + version;
        var data = [idStm, geometryStm, versionStm].join('&');

        //
        var request = YAHOO.util.Connect.asyncRequest('POST', gt.config.httpPath + "/editgeom/get", getCallback, data);
    },

    getSuccess: function (o) {
        
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        } 
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }

        var setBodySuccess = gt.accord.setBody(this.getPanelIndex(), responseText.form);
        if(! setBodySuccess) {
            return;
        }
        
        if(responseText.geometry != undefined) {
            //Unselect
            gt.control.editgeom.unselectAll();
            
            //Change geometry
            gt.layer.edit.removeFeatures(this.feature, true);
            this.feature.geometry.destroy();
            this.feature.geometry = new OpenLayers.Geometry.fromWKT(responseText.geometry);
            gt.layer.edit.addFeatures(this.feature, true);
            
            this.feature.state = OpenLayers.State.UPDATE;
            
            //Select
            gt.control.editgeom.controls.editControl.selectFeature(this.feature);
        } 
        
        this.initInput();
        
        //Trigger form render event
        gt.control.editgeom.events.triggerEvent("formrender", YAHOO.util.Dom.get('gtEditattrForm'));
        
    },

    submitSuccess: function (o) {

        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        } 
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        
        if (! responseText.success) {
            if (responseText.errors) {
                for (var error in responseText.errors) {
                    Giscuit.Util.message(responseText.errors[error]);
                }
                if(gt.control.editgeom.controls.editControl.active) {
                    gt.accord.setBody(this.getPanelIndex(), responseText.form);
                    this.initInput();
                } else {
                    gt.control.editgeom.unselectAll();
                    gt.control.editgeom.clearInsertedFeatures();
                    this.destroy();
                }
            } else {
                gt.accord.setBody(this.getPanelIndex(), responseText.form);
                this.initInput();
            }
            
        } else {
            //Submit
            
            //Destroy original
            if(this.feature.originalGeometry != undefined) {
                this.feature.originalGeometry.destroy();
                this.feature.originalGeometry = undefined;
            }
            this.feature.state = null;
            
            gt.control.editgeom.unselectAll();
                            
            if(responseText.gid != undefined) {
                this.feature.attributes.gid = responseText.gid;
            }

            gt.control.editgeom.backActivate();

            //Update attributes
            var values = YAHOO.util.Connect.setForm('gtEditattrForm').split('&');
            for(var i=0; i < values.length; i++) {
                var key = decodeURIComponent(values[i].split('=')[0]);
                var value = decodeURIComponent(values[i].split('=')[1]);
                
                this.feature.attributes[key] = value;
            }

            //Trigger add/edit event with feature attributes
            var data = {
                values: this.feature.attributes,
                feature: this.feature
            };
            
            if (typeof (responseText.gid) != 'undefined') {
                gt.control.editgeom.events.triggerEvent("addfeature", data);
            } else {
                gt.control.editgeom.events.triggerEvent("editfeature", data);
            }
            
            //FIXME:
            gt.layer.edit.removeFeatures(this.feature, true);
            gt.layer.edit.addFeatures(this.feature, true);
                            
            this.destroy();
        }

    },

    submit: function () {
        //HACK: Enable all file elements
        var nodes = YAHOO.util.Selector.query('#gtEditattrPH input[type=file][disabled=disabled]');
        for(var i = 0; i < nodes.length; i++) {
            nodes[i].removeAttribute('disabled');
        }
                
        this.submitButton.set('disabled', true);

        var submitCallback = {
            upload: this.submitSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };
        
        var postData = '';
        if(this.feature.geometry != null) {
            postData = 'geometry=' + this.feature.geometry;
        }
        
        var stm = '';
        if(this.feature.attributes.gid != null) {
            stm = '/edit/id/' + this.feature.attributes.gid;
        } else {
            stm = '/add';
        }
        
        YAHOO.util.Connect.setForm('gtEditattrForm', true);
        YAHOO.util.Connect.asyncRequest('POST', gt.config.httpPath + "/editgeom" + stm, submitCallback, postData);

    },

    initInput: function () {
        var dom = YAHOO.util.Dom;
        var self = this;

        //
        var editattrPH = dom.get('gtEditattrPH');
        
        //Submit
        this.submitButton = new YAHOO.widget.Button('gtEditattrSubmit');
        this.submitButton.on('click', this.submit, null, this);
        
        //Cancel
        this.cancelButton = new YAHOO.widget.Button('gtEditattrCancel');
        this.cancelButton.on('click', function() {
            gt.control.editgeom.unselectAll();
            gt.control.editgeom.clearInsertedFeatures();
            this.destroy();
        }, null, this);
        
        //
        var versionSelect = dom.get('gtEditattrVersionsSelect');
        if(versionSelect) {
            this.versionsSelect = new YAHOO.widget.Button("gtEditattrVersionsButton", { 
                label: versionSelect[versionSelect.selectedIndex].text ,
                type: "menu",  
                menu: "gtEditattrVersionsSelect" 
            }); 
            this.versionsSelect.getMenu().subscribe("click", function(evt, data) {
                this.changeVersion(data[1].value);
            }, null, this);
        }

        //Date
        var nodes = YAHOO.util.Selector.query('.gt-field-date', editattrPH);
        for(var i = 0; i < nodes.length; i++) {
            Giscuit.Util.text2cal(nodes[i], this.dateInstances);
        }
        
        //File
        var nodes = YAHOO.util.Selector.query('input[type="file"]', editattrPH);
        for(var i = 0; i < nodes.length; i++) {
            if(! nodes[i].nextSibling) {
                continue;
            }
            
            YAHOO.util.Event.addListener(nodes[i].nextSibling, "click", function(evt, fileEl) {
                if(confirm(gt.translate._('deleteConfirm'))) {
                    dom.setAttribute(fileEl, 'disabled', 'disabled');
                    dom.setAttribute(fileEl.previousSibling, 'value', '');
                }
            }, nodes[i], this);
        }
        
        //Fieldset
        var nodes = YAHOO.util.Selector.query('fieldset', editattrPH);
        for(var i = 0; i < nodes.length; i++) {
            var legend = nodes[i].getElementsByTagName('legend')[0];

            //Hide label
            nodes[i].parentNode.previousSibling.style.display = 'none';

            //Status
            var status = 'off';
            if(dom.getElementsByClassName('errors', 'ul', nodes[i]).length) {
                var status = 'on';
            }
            dom.addClass(nodes[i], status);
            
            //Add span
            var toggleSpan = document.createElement('span');
            legend.appendChild(toggleSpan);
        }
        YAHOO.util.Event.delegate('gtEditattrPH', "click", function(evt, matchedEl, container) {
            var fieldset = matchedEl.parentNode;
            
            //
            var status = 'on';
            if(dom.hasClass(fieldset, 'on')) {
                status = 'off';
            } 
            
            //Reset
            var nodes = YAHOO.util.Selector.query('fieldset', editattrPH);
            for(var i = 0; i < nodes.length; i++) {
                dom.removeClass(nodes[i], 'on');
                dom.addClass(nodes[i], 'off');
            }

            //Set
            dom.addClass(fieldset, status);
                        
        }, "legend");

        //Subform - Hide elements with class "gtSubformDefault"
        var nodes = YAHOO.util.Selector.query('.gtSubformDefault', editattrPH);
        var i = nodes.length;
        while(i--) {
            var el = nodes[i];

            if(el.tagName.toUpperCase() == 'DD') {
                var dd = el;
                var dt = dom.getPreviousSibling(dd);

                dd.style.display = 'none';
                dt.style.display = 'none';
            } else {
                el.style.display = 'none';
            }
        }

        //Subform - Insert add buttons
        var nodes = YAHOO.util.Selector.query('fieldset[subform=1] dl', editattrPH);
        var i = nodes.length;
        while(i--) {
            var el = nodes[i];

            var addImg = document.createElement('img');
            addImg.className = 'gtSubformAdd';
            addImg.src = gt.config.httpImagePath + '/list_add.png';
            addImg.title = gt.translate._('add');

            el.appendChild(addImg);
        }

        //Subform - Events
        YAHOO.util.Event.delegate('gtEditattrPH', "click", function(evt, matchedEl, container) {
            //Del
            if(dom.hasClass(matchedEl, 'gtSubformDel')) {
                var delEl = this;
                var currEl = delEl;

                //Delete all elements until we hit a HR
                while(currEl.tagName.toUpperCase() != 'HR') {
                    var lastEl = currEl;
                    currEl = dom.getPreviousSibling(currEl);
                    lastEl.parentNode.removeChild(lastEl);
                }
                currEl.parentNode.removeChild(currEl);
            }

            //Add
            if(dom.hasClass(matchedEl, 'gtSubformAdd')) {
                var addEl = this;
                var fieldset = addEl.parentNode.parentNode;

                //Get index
                var idx = parseInt(fieldset.getAttribute('data-count'));

                var nodes = YAHOO.util.Selector.query('#' + fieldset.id + ' .gtSubformDefault');
                for(var i = 0; i < nodes.length; i++) {
                    var el = nodes[i];

                    if(el.tagName.toUpperCase() == 'DD') {
                        var dateFields = [];

                        var dd = el;
                        var dt = dom.getPreviousSibling(dd);

                        //Clone
                        var newDd = dd.cloneNode(true);
                        var newDt = dt.cloneNode(true);

                        //Remove default class
                        dom.removeClass(newDd, 'gtSubformDefault');

                        //Set name
                        var input = YAHOO.util.Selector.query('input, textarea, select, file', newDd);
                        for(var j = 0; j < input.length; j++) {

                            //Multi fix
                            if(input[j].name[input[j].name.length - 1] == ']'
                                && input[j].name[input[j].name.length - 2] == '[') {

                                input[j].name = input[j].name.substring(0, input[j].name.length - 2) + '_' + idx + '[]';
                            } else {
                                input[j].name = input[j].name + '_' + idx;
                            }

                            //Date fix
                            if(dom.hasClass(input[j], 'gt-field-date')) {
                                input[j].id = input[j].name;
                                input[j].parentNode.removeChild(dom.getNextSibling(input[j]));
                                input[j].parentNode.id = input[j].id + '-element';

                                Giscuit.Util.text2cal(input[j], self.dateInstances);
                            }
                        }

                        //Set visibility
                        newDd.style.display = 'block';
                        newDt.style.display = 'block';

                        //Insert
                        dom.insertBefore(newDt, addEl);
                        dom.insertBefore(newDd, addEl);

                    //Add hr and img
                    } else {
                        var newEl = el.cloneNode(true);
                        newEl.style.display = 'block';
                        dom.removeClass(newEl, 'gtSubformDefault');
                        dom.insertBefore(newEl, addEl);
                    }
                }

                //Set index
                idx++;
                fieldset.setAttribute('data-count', idx)
            }
        }, "img");

        //
        var heightCoef = 30;
        dom.get('gtEditattrPH').style.maxHeight = gt.config.accordPanelMaxHeight - heightCoef + 'px';
    },
    
    changeVersion: function(id) {
        this.initialize(null, id);
    },
    
    destroy: function() {
        //
        if(this.submitButton != null) {
            this.submitButton.destroy();
            this.submitButton = null;
        }

        //
        if(this.cancelButton != null) {
            this.cancelButton.destroy();
            this.cancelButton = null;
        }

        //
        if(this.versionsSelect != null) {
            this.versionsSelect.destroy();
            this.versionsSelect = null;
        }

        //
        if(this.panel) {
            YAHOO.util.Event.purgeElement(this.panel, true);
            gt.accord.removePanel(this.getPanelIndex());
        }
        this.panel = null;

        //
        if(this.dateInstances) {
            for (var i = this.dateInstances.length - 1; i >= 0; i--) {
                try {
                    this.dateInstances[i].destroy();
                } catch(evt) {
                }
                this.dateInstances[i] = null;
            }
            this.dateInstances = null;
        }
        
        OpenLayers.Control.prototype.destroy.call(this);
    },
    
    CLASS_NAME: "Giscuit.Control.Editattr"
});
