/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Measure.js 2844 2012-07-09 08:54:35Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        if(gt.config.layout != 'desktop') {
            return;
        }
        
        gt.control.measure = new Giscuit.Control.Measure( {
            type : 'distance'
        });
        
        gt.control.area = new Giscuit.Control.Measure( {
            type : 'area'
        });
    }
});

/**
 * Class: Giscuit.Control.Measure
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Measure = OpenLayers.Class(Giscuit.Control, {

    control: null,

    panel: null,
    
    button: null,
    
    type: 'distance',
    
    displaySystemUnits: {
        geographic: ['dd'],
        english: ['mi', 'ft', 'in'],
        metric: ['km', 'm']
    },
    
    tooltipShown: false,
    
    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        //
        if(this.type == 'area') {
            this.button = gt.control.measure.button;
            return;
        }
        
        var self = this;
        
        var onClick = function (type, args, item) {
            var label = item.cfg.getProperty('text');
            self.button.set('label', label);

            if(this.id == 'gtControlMeasure') {
                if(gt.control.measure.active) {
                    gt.control.measure.deactivate();
                } else {
                    gt.control.measure.activate();
                }
            } else if(this.id == 'gtControlArea') {
                if(gt.control.area.active) {
                    gt.control.area.deactivate();
                } else {
                    gt.control.area.activate();
                }
            }
        };
        
        var menu = [{
            id: 'gtControlMeasure' ,
            text: gt.map.getControlIcon(13, gt.translate._('measureDistance')),
            onclick: {fn: onClick}
        }, {
            id: 'gtControlArea' ,
            text: gt.map.getControlIcon(1, gt.translate._('measureArea')),
            onclick: {fn: onClick}
        }];
        
        this.button = new YAHOO.widget.Button({
            type: "menu", 
            label: menu[0].text,
            title: gt.translate._('measure'),
            'menu': menu, 
            container: "gtMapControls",
            lazyloadmenu: false
        });
    },

    activate: function () {
        var deactivateEditgeom = false;
        if(typeof(gt.control.editgeom) != 'undefined' && gt.control.editgeom.active) {
            deactivateEditgeom = gt.control.editgeom.active;
        } 
        
        gt.map.deactivateAll();
        
        var activated = Giscuit.Control.prototype.activate.call(this);
                
        if (activated == true) {
            
            //Check button
            if(this.type == 'distance') {
                this.button.set("label", Giscuit.Util.getMenuItemById(this.button, 'gtControlMeasure').cfg.getProperty('text'));
            }  else if(this.type == 'area') {
                this.button.set("label", Giscuit.Util.getMenuItemById(this.button, 'gtControlArea').cfg.getProperty('text'));
            }
            YAHOO.util.Dom.addClass(this.button.get('element'), 'yui-checkbox-button-checked');
            
            //Tooltip
            if (! this.tooltipShown && gt.config.tooltip == 't') {
                gt.tooltip.cfg.setProperty('text', gt.translate._('drawGeometryHelp'));
                gt.tooltip.showTooltip();
                this.tooltipShown = true;
            }
            
            if(this.type == 'distance') {
                var handler = OpenLayers.Handler.Path;
            } else if(this.type == 'area') {
                var handler = OpenLayers.Handler.Polygon;
            }
            
            this.control = new OpenLayers.Control.DrawFeature(gt.layer.vector, handler, {
                callbacks: {
                    "point": OpenLayers.Function.bind(this.populate, this),
                    "done": this.done
                },
                handlerOptions: {
                    persist: true
                }
            });
            gt.map.addControl(this.control);
            this.control.activate();
            
            //Activate snap
            var activateSnap = function() {
                
                if(typeof(gt.layer.snap) != 'undefined') {
                    gt.layer.snap.setVisibility(true);
                }
                if (typeof(gt.snap.vector) != 'undefined') {
                    gt.snap.vector.activate();
                }
            };
            //FIXME: Postpone snap layer activation
            if(deactivateEditgeom) {
                setTimeout(function() {
                    activateSnap();
                }, 1000);
            } else {
                activateSnap();
            }
        }

        return activated;
    },

    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);

        if (deactivated == true) {
            this.control.destroy();
            this.control = null;
            
            //Deactivate snap
            if (typeof(gt.snap.vector) != 'undefined') {
                gt.snap.vector.deactivate();
            }
            if(typeof(gt.layer.snap) != 'undefined') {
                gt.layer.snap.setVisibility(false);
            }
            
            //Uncheck button
               YAHOO.util.Dom.removeClass(
                   this.button.get('element'), 
                   'yui-checkbox-button-checked'
               );
        }
        
        return deactivated;
    },

    getBestLength: function (geometry) {
        var units = this.displaySystemUnits[gt.config.displayUnits];
        
        var myLength, geomUnits;
        
        if(geometry.getVertices().length > 2) {
            geometry.removeComponent(geometry.getVertices()[geometry.getVertices().length - 1]);
        } else {
            return [0, units[units.length - 1]];
        }
        
        var unit, myLength;
        for(var i=0, len=units.length; i<len; ++i) {
            unit = units[i];
            myLength = this.getLength(geometry, unit);
            if(myLength > 1) {
                break;
            }
        }
        
        geometry.destroy();
        geometry = null;
        
        return [myLength, unit];
    },
    
    getLength: function(geometry, units) {
        
        var wgsProj = new OpenLayers.Projection("EPSG:4326");
        var wgsGeometry = geometry.clone();

        wgsGeometry.transform(gt.map.getProjectionObject(), wgsProj); 
       
        var myLength = wgsGeometry.getGeodesicLength(wgsProj);
        geomUnits = "m";
       
        var inPerDisplayUnit = OpenLayers.INCHES_PER_UNIT[units];
        if(inPerDisplayUnit) {
            var inPerMapUnit = OpenLayers.INCHES_PER_UNIT[geomUnits];
            myLength *= (inPerMapUnit / inPerDisplayUnit);
        }

        return myLength;
    },

    
    getBestArea: function(geometry) {
        var units = this.displaySystemUnits[gt.config.displayUnits];
        
        if(geometry.getVertices().length > 3) {
            geometry.removeComponent(geometry.getVertices()[geometry.getVertices().length - 1]);
        } else {
            return [0, units[units.length - 1]];
        }
        
        var unit, area;
        for(var i=0, len=units.length; i<len; ++i) {
            unit = units[i];
            area = this.getArea(geometry, unit);
            if(area > 1) {
                break;
            }
        }
        return [area, unit];
    },

    getArea: function(geometry, units) {
        var wgsProj = new OpenLayers.Projection("EPSG:4326");
        var wgsGeometry = geometry.clone();

        wgsGeometry.transform(gt.map.getProjectionObject(), wgsProj);
        
        area = wgsGeometry.getGeodesicArea(wgsProj);
        geomUnits = "m";
        
        var inPerDisplayUnit = OpenLayers.INCHES_PER_UNIT[units];
        if(inPerDisplayUnit) {
            var inPerMapUnit = OpenLayers.INCHES_PER_UNIT[geomUnits];
            area *= Math.pow((inPerMapUnit / inPerDisplayUnit), 2);
        }
        return area;
    },

    populate: function (current, geometry) {
        
        if(this.type == 'distance') {
            var result = this.getBestLength(geometry.clone());
            var text = gt.translate._('distance') + ": " + result[0].toFixed(3) + ' ' + result[1];                                                                  ;
            var label = 'measureDistance';
        } else if(this.type == 'area') {
            var result = this.getBestArea(geometry);
            var text = gt.translate._('area') + ": " + result[0].toFixed(3) + ' ' + result[1] + '<sup>2</sup>';
            var label = 'measureArea';
        }

        if(this.panel == null) {
            gt.accord.addPanel({
                label: gt.translate._(label),
                content: ''
            });
            this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
            this.addHelpButton('http://docs.giscuit.com/En/tutorial_refguide.user.desktop.measure.pkg.html');
        } 
        
        gt.accord.setBody(this.getPanelIndex(), text);
        gt.accord.openPanel(this.getPanelIndex());
         
    },

    done: function () {
        
    },
    
    CLASS_NAME: "Giscuit.Control.Measure"

});