/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Print.js 2744 2012-06-12 07:27:23Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.quickaccess = new Giscuit.Control.Quickaccess();
    }
});

/**
 * Class: Giscuit.Control.Quickaccess
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Quickaccess = OpenLayers.Class(Giscuit.Control, {
    
    button: null,
    
    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);

        //Create zones menu
        var records = gt.pluginParams.quickaccessRecords;
        var adminItems = [];
        var userItems = [];
        for (var i in records) {
            var item = {
                id: records[i].gid,
                text: records[i].name,
                value: records[i].the_geom,
                onclick: { fn: this.click }
            }
            if (records[i].published == 't') {
                adminItems.push(item);
            } else {
                userItems.push(item);
            }
        }

        //Create menu button
        this.button = new YAHOO.widget.Button({
            type: 'menu', 
            label: gt.map.getControlIcon(34, gt.translate._('quickaccess')),
            'menu': [adminItems, userItems],
            title: gt.translate._('quickaccess'), 
            container: 'gtMapControls',
            lazyloadmenu: false
        });  

        //Add custom menu style
        this.button.getMenu().element.className += ' quickaccess';
        
        if (gt.config.guest == 'f') {
            //Register for create event
            gt.control.editgeom.events.register("addfeature", this, function(data) {
                //Check the editable layer
                if (gt.control.layers.getEditLayer()['class'] != 'Quickaccess') {
                    return;
                }
                
                this.addRecord(data);
            });
            
            //Register for update event
            gt.control.editgeom.events.register("editfeature", this, function(data) {
                //Check the editable layer
                if (gt.control.layers.getEditLayer()['class'] != 'Quickaccess') {
                    return;
                }
                
                this.deleteRecord(data.values.gid);
                this.addRecord(data);
            });
            
            //Register for delete event
            gt.control.editgeom.events.register("deletefeature", this, function(gid) {
                //Check the editable layer
                if (gt.control.layers.getEditLayer()['class'] != 'Quickaccess') {
                    return;
                }
                
                this.deleteRecord(gid);
            });
        }
    },
    
    click: function(type, args, item) {
        var geometry = OpenLayers.Geometry.fromWKT(item.value);
        gt.map.zoomToExtent(geometry.getBounds());
    },
    
    deleteRecord: function(gid) {
        var menu = this.button.getMenu();
        var menuItemGroups = menu.getItemGroups();
        for (var i = 0; i < menuItemGroups.length; i++) {
            for (var y = 0; y < menuItemGroups[i].length; y++) {
                if (menuItemGroups[i][y].id == parseInt(gid)) {
                    menu.removeItem(y, i);
                    return;
                }
            }
        }
    },
    
    addRecord: function(data) {
        var name = data.values['name_' + gt.config.locale];
        var menu = this.button.getMenu();
        
        if (data.values.published == 't') {
            var menuGroup = 0;
        } else {
            var menuGroup = 1;
        }
        
        menu.addItem({
            id: data.values.gid,
            text: name,
            value: data.feature.geometry.toString(),
            onclick: { fn: this.click }
        }, menuGroup);
        
        menu.render();
    },
    
    CLASS_NAME: "Giscuit.Control.Quickaccess"
});