/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Datatable.js 1749 2011-09-15 11:10:45Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Init
 * Contains giscuit initialization code
 */ 
Giscuit.Init = function() {
    this.layoutStorage = null;
    this.layout = null;
    this.centerViewResize = null;
    this.centerTopEl = null;
    this.centerBottomEl = null;
        
    this.initializeStorage();
    this.initializeLayout();
    this.initializeTopPanel();
    this.initializeAccordion();
    this.initializeAccordionLeftPanel();
        
    YAHOO.util.Dom.get('gtLoading').style.display = 'none';
    YAHOO.util.Dom.get('gtTop').style.display = 'block';
    YAHOO.util.Dom.get('gtLeft').style.display = 'block';
    YAHOO.util.Dom.get('gtCenter').style.display = 'block';
};
    
Giscuit.Init.prototype.initializeStorage = function() {
    debug.info('Initializing storage');
    
    this.layoutStorage = Giscuit.Util.getCookie('gtLayout');
    if(this.layoutStorage === null) {
        this.layoutStorage = {
            leftOpened: true,
            topOpened: true,
            leftWidth: 385
        };
    } else {
        this.layoutStorage = YAHOO.lang.JSON.parse(this.layoutStorage);
    }
    
    if(typeof(this.leftOpened) != 'undefined') {
        this.layoutStorage.leftOpened = this.leftOpened;
    }
    if(typeof(this.topOpened) != 'undefined') {
        this.layoutStorage.topOpened = this.topOpened;
    }
    if(typeof(this.leftWidth) != 'undefined') {
        this.layoutStorage.leftWidth = this.leftWidth;
    }
    
    if(typeof(this.save) == 'undefined') {
        this.save = true;
    }
    this.save = !!this.save;
};

Giscuit.Init.prototype.openCenterView = function(position, coef) {
    if(YAHOO.util.Dom.get('gtCenterView')) {
        return false;
    }
    
    var position = position == 'top' ? 'top' : 'bottom';
    if(! coef) {
        coef = 0.6;
    }
    var centerRegion = YAHOO.util.Dom.getRegion('gtCenter');
    
    //FIXME:
    if(! centerRegion.height) {
        centerRegion.height = this.layout.getUnitByPosition('center').get('height');
    }
    
    //centerView
    var centerView = document.createElement('div');
    centerView.id = 'gtCenterView';
    if(position == 'top') {
        YAHOO.util.Dom.insertBefore(centerView, 'gtMap');
    } else if (position == 'bottom') {
        YAHOO.util.Dom.insertAfter(centerView, 'gtMap');
    }
    
    //
    if(position == 'top') {
        this.centerBottomEl = YAHOO.util.Dom.get('gtMap');
        this.centerTopEl = centerView;
    } else if (position == 'bottom') {
        this.centerBottomEl = centerView;
        this.centerTopEl = YAHOO.util.Dom.get('gtMap');
    }
    
    this.centerBottomEl.style.borderTop = '1px solid gray';
    
    //
    this.centerTopEl.style.height = parseInt(centerRegion.height * coef) + 'px';
    this.centerBottomEl.style.height = parseInt(centerRegion.height * (1 - coef)) + 'px';
    var mapRegion = YAHOO.util.Dom.getRegion('gtMap');
    var mapPosY = mapRegion.y - centerRegion.y; 
    if(mapPosY) {
        YAHOO.util.Dom.get('gtMapControls').style.top = mapPosY  + 'px';
    }
    
    //Resize
    var resizeDiv = document.createElement('div');
    resizeDiv.id = 'gtCenterViewResize';
    resizeDiv.className = 'yui-resize-handle yui-resize-handle-t';
    resizeDiv.innerHTML = '<div class="yui-layout-resize-knob"></div>';
    YAHOO.util.Dom.insertBefore(resizeDiv, YAHOO.util.Dom.get('gtCenter').firstChild);
    resizeDiv.style.top = YAHOO.util.Dom.getRegion(this.centerTopEl).height - 5 + 'px';
    this.centerViewResize = new Giscuit.Util.DDRegion(resizeDiv, '', {
        cont: YAHOO.util.Dom.get('gtCenter')
    });
    
    //Map
    gt.map.updateSize();
    gt.map.refresh();
    
    return true;
};

Giscuit.Init.prototype.closeCenterView = function() {
    if(! YAHOO.util.Dom.get('gtCenterView')) {
        return false;
    }
    
    this.centerBottomEl.style.borderTop = 0;
    YAHOO.util.Dom.get('gtMapControls').style.top = '-1px';
    
    this.centerTopEl = null;
    this.centerBottomEl = null;
    
    var centerView = YAHOO.util.Dom.get('gtCenterView');
    centerView.parentNode.removeChild(centerView);
    
    this.centerViewResize = null;
    var centerViewResize = YAHOO.util.Dom.get('gtCenterViewResize');
    centerViewResize.parentNode.removeChild(centerViewResize);
    
    gt.init.resize();
    
    return true;
};

Giscuit.Init.prototype.initializeLayout = function() {
    debug.info('Initializing layout');
        
    this.layout = new YAHOO.widget.Layout({
        units:[{
            position: 'top',
            body: 'gtTop',
            gutter: '0',
            scroll: null,
            collapseSize: 0,
            animate: false,

            height: 90
        },{
            position: 'left',
            body: 'gtLeft',
            gutter: '3px 6px 3px 3px',
            scroll: null,
            collapseSize: -9,
            animate: false,
            
            width: this.layoutStorage.leftWidth, 
            //FIXME:
            footer: '<div style="padding:3px;">' + gt.translate._('poweredByGiscuit') + ' ' + gt.config.version + '<div>',
            resize: true
        },{
            position: 'center',
            body: 'gtCenter',
            gutter: '3px',
            scroll: null
        }]
    });
    
    this.layout.render();
    
};

Giscuit.Init.prototype.initializeAccordion = function() {
    debug.info('Initializing accordion');
    
    var layoutStorage = this.layoutStorage;
    var self = this;

    gt.accord = new YAHOO.widget.AccordionView('gtAccordion', {
        collapsible: true, 
        expandable: false, 
        width: (self.layout.getUnitByPosition('left').getSizes().body.w + 2) + 'px', 
        animate: false
        //animationSpeed: '0.2'
    });

    gt.accord.on('panelOpen', function(oArgs) {
        var node = YAHOO.util.Dom.getElementsByClassName('yui-accordion-content', 'div', oArgs.panel)[0];
        node.style.visibility = 'visible';
        node.style.overflow = 'hidden';
       
        //
        var gtPrivatelayersCategoriesPH = YAHOO.util.Dom.get('gtPrivatelayersCategoriesPH');        
        if(gtPrivatelayersCategoriesPH && gtPrivatelayersCategoriesPH.parentNode == node) {
            gtPrivatelayersCategoriesPH.style.visibility = 'visible';
        }
        
        //
        self.accordPanelMaxHeight();
    });
    
    var firstPanelOpen = false;
    gt.accord.on('afterPanelOpen', function(oArgs) {
        var node = YAHOO.util.Dom.getElementsByClassName('yui-accordion-content', 'div', oArgs.panel)[0];
        
        node.style.overflowY = 'auto';
        node.style.overflowX = 'hidden';
        
        //Set max height
        if (YAHOO.env.ua.ie && YAHOO.env.ua.ie <= 6) {
            if(node.onresize == undefined) {
                node.onresize = function() {
                    var region = YAHOO.util.Dom.getRegion(this);
                    if(region.height > gt.config.accordPanelMaxHeight) {
                        this.style.height = gt.config.accordPanelMaxHeight + 'px';  
                    }
                };
            }
        } else {
            node.style.maxHeight = gt.config.accordPanelMaxHeight + 'px';
        }
        
        if(! firstPanelOpen) {
            firstPanelOpen = true;
        } else {
            self.layout.getUnitByPosition('left').expand();
        }
    });
    
    gt.accord.on('panelClose', function(oArgs) {
        var node = YAHOO.util.Dom.getElementsByClassName('yui-accordion-content', 'div', oArgs.panel)[0];
        node.style.overflow = 'hidden';
        
        var gtPrivatelayersCategoriesPH = YAHOO.util.Dom.get('gtPrivatelayersCategoriesPH');  
        if(gtPrivatelayersCategoriesPH) {
            gtPrivatelayersCategoriesPH.style.visibility = 'hidden';
        }
    });
    
    gt.accord.on('afterPanelClose', function(oArgs) {
        var node = YAHOO.util.Dom.getElementsByClassName('yui-accordion-content', 'div', oArgs.panel)[0];
        node.style.visibility = 'hidden';
    });
    
    //Resize accordion when left panels resizes
    this.layout.getUnitByPosition('left').on('widthChange', function(args) {
        gt.accord.get('element').style.width = this.getSizes().body.w + 2 + 'px';
    });
    
    this.layout.getUnitByPosition('left').on('collapse', function() {
        var clip = YAHOO.util.Dom.getElementsByClassName('yui-layout-clip-left', 'div')[0];
        clip.style.visibility = 'hidden';
        clip.children[0].style.visibility = 'visible';
        clip.children[0].style.left = '-1px';
        clip.children[0].style.top = '24px';
        
        layoutStorage.leftOpened = false;
        if(self.save) {
            Giscuit.Util.setCookie('gtLayout', YAHOO.lang.JSON.stringify(layoutStorage));
        }
    });
    
    this.layout.getUnitByPosition('left').on('expand', function() {
        layoutStorage.leftOpened = true;
        if(self.save) {
            Giscuit.Util.setCookie('gtLayout', YAHOO.lang.JSON.stringify(layoutStorage));
        }
    });
    
    this.layout.getUnitByPosition('left').on('resize', function(oArgs) {
        if(! this.get('width')) {
            return;
        }

        if(this.get('width') < 255) {
            this.set('width', 255);
        }
        
        layoutStorage.leftWidth = this.get('width');
        if(self.save) {
            Giscuit.Util.setCookie('gtLayout', YAHOO.lang.JSON.stringify(layoutStorage));
        }
    });
    
    if(! layoutStorage.leftOpened) {
        this.layout.getUnitByPosition('left').collapse();
    }
    
};

Giscuit.Init.prototype.accordPanelMaxHeight = function() {
    //FIXME: Static
    var totalHeight = 40;

    var panels = gt.accord.getPanels();
    for(var i=0; i < panels.length; i++) {
        var region = YAHOO.util.Dom.getRegion(panels[i].children[0]);
        totalHeight += region.height;
    }
    
    gt.config.accordPanelMaxHeight = this.layout.getUnitByPosition('left').get('height') - totalHeight;
    
    if(gt.config.accordPanelMaxHeight < 250) {
        gt.config.accordPanelMaxHeight = 250;
    }   
};

Giscuit.Init.prototype.initializeAccordionLeftPanel = function() {
    debug.info('Initializing accordion first left panel');
    
    var self = this;
    var layoutStorage = this.layoutStorage;
        
    if(! gt.accord.getPanels()) {
        return;
    } 
    
    //Toggle
    gt.accord.getPanel(0).firstChild.firstChild.nextSibling.style.display = 'none';
    
    var expandButton = document.createElement('div');
    expandButton.id = 'gtLeftPanelToggle';
    expandButton.title = gt.translate._('hidePanel');
    
    YAHOO.util.Dom.insertAfter(expandButton,
        gt.accord.getPanel(0).firstChild.firstChild.nextSibling
    );
    YAHOO.util.Event.addListener(expandButton, 'click', function() {
        self.layout.getUnitByPosition('left').collapse();
    });
    
    //Open
    gt.accord.openPanel(0);
};

Giscuit.Init.prototype.initializeTopPanel = function() {
    debug.info('Initializing top panel');
    
    var self = this;
    var layoutStorage = this.layoutStorage;
    
    //Toggle
    YAHOO.util.Event.addListener('gtTopPanelToggle', 'click', function() {
        self.layout.getUnitByPosition('top').collapse();
    });
    this.layout.getUnitByPosition('top').on('collapse', function() {
        var clip = YAHOO.util.Dom.getElementsByClassName('yui-layout-clip-top', 'div')[0];
        clip.style.visibility = 'hidden';
        clip.children[0].style.visibility = 'visible';
        clip.children[0].style.backgroundPosition = '-20px -240px';
        
        layoutStorage.topOpened = false;
        Giscuit.Util.setCookie('gtLayout', YAHOO.lang.JSON.stringify(layoutStorage));
    });
    
    this.layout.getUnitByPosition('top').on('expand', function() {
        layoutStorage.topOpened = true;
        Giscuit.Util.setCookie('gtLayout', YAHOO.lang.JSON.stringify(layoutStorage));
    });
    
    if(! layoutStorage.topOpened) {
        this.layout.getUnitByPosition('top').collapse();
    }
};

Giscuit.Init.prototype.resize = function() {
    YAHOO.util.Dom.get('gtMap').style.width = (this.layout.getUnitByPosition('center').get('width') - 8)  + 'px';
    var centerHeight = this.layout.getUnitByPosition('center').get('height');
        
    if(YAHOO.util.Dom.get('gtCenterView')) {
        var centerViewRegion = YAHOO.util.Dom.getRegion('gtCenterView');
        YAHOO.util.Dom.get('gtMap').style.height = centerHeight - centerViewRegion.height + 'px';
    } else {
        YAHOO.util.Dom.get('gtMap').style.height = centerHeight - 8 + 'px';
    }
    
    //Set accordion panel max and min height 
    gt.config.accordPanelMaxHeight = this.layout.getUnitByPosition('left').get('height') - 180;
    if(gt.config.accordPanelMaxHeight < 300) {
        gt.config.accordPanelMaxHeight = 300;
    }
    
    gt.map.updateSize();
};

(function() {
    var Dom = YAHOO.util.Dom,
        Event = YAHOO.util.Event;

    Giscuit.Util.DDRegion = function(id, sGroup, config) {
        this.cont = config.cont;
        Giscuit.Util.DDRegion.superclass.constructor.apply(this, arguments);
    };

    YAHOO.extend(Giscuit.Util.DDRegion, YAHOO.util.DD, {
        
        cont: null,
        
        init: function() {
            Giscuit.Util.DDRegion.superclass.init.apply(this, arguments);
            this.initConstraints();

            Event.on(window, 'resize', function() {
                this.initConstraints();
            }, this, true);
        },
        
        initConstraints: function() {
            var region = Dom.getRegion(this.cont);
            var el = this.getEl();
            var xy = Dom.getXY(el);
            
            var width = parseInt(Dom.getStyle(el, 'width'), 10);
            var height = parseInt(Dom.getStyle(el, 'height'), 10);

            var left = xy[0] - region.left;
            var right = region.right - xy[0] - width;

            var top = xy[1] - region.top;
            var bottom = region.bottom - xy[1] - height;

            this.setXConstraint(left, right);
            this.setYConstraint(top, bottom  - 5);
        },
        
        startDrag: function(x, y) {
            Dom.addClass(this.getEl(), 'resizing');
        },

        endDrag: function(e) {
            Dom.removeClass(this.getEl(), 'resizing');
            
            //
            var thisRegion = Dom.getRegion(this.getEl());
            var centerRegion = Dom.getRegion(Dom.get('gtCenter').parentNode);
            var thisY = thisRegion.top - centerRegion.top;
            
            gt.init.centerTopEl.style.height = thisY + 'px';
            gt.init.centerBottomEl.style.height = centerRegion.height - thisY + 'px';
            
            var mapRegion = YAHOO.util.Dom.getRegion('gtMap');
            var mapPosY = mapRegion.y - centerRegion.y; 
            if(mapPosY) {
                YAHOO.util.Dom.get('gtMapControls').style.top = mapPosY - 1  + 'px';
            }
            
            gt.map.updateSize();
        }
    });
})();

