/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Editattr.js 1555 2011-08-02 15:41:37Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Mobile.Control.Editattr
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Mobile.Control.Editattr = OpenLayers.Class(Giscuit.Control, {

    feature: null,

    initialize: function (feature, version) {
        if(feature != null) {
            this.feature = feature;
        }
        
        if(gt.control.offline.isOnline) {
             $.mobile.showPageLoadingMsg();

            //
            var idStm = '';
            if(this.feature.attributes.gid != null) {
                idStm = "id=" + this.feature.attributes.gid;
            }
            var geometryStm = '';
            if(this.feature.geometry != null) {
                geometryStm = "geometry=" + encodeURIComponent(this.feature.geometry.toString());
            }
            var versionStm = 'version=' + version;
            var data = [idStm, geometryStm, versionStm].join('&');

            //
            var url = gt.config.httpMobilePath + "/editgeom/get";
            $.ajax({
                type: "POST",
                dataType: "json",
                url: url,
                data: data,
                success: Giscuit.Util.bind(this.getSuccess, this)
            })
                .error(Giscuit.Util.jqmAjaxError);

        } else {
            var editForm = Giscuit.Util.getStorageObject('gtEditForm');
            var formHtml = editForm['layer_' + gt.control.layers.getEditLayer().id];
            this.getSuccess({
                'form': formHtml
            });
        }
    },
    
    getSuccess: function (data) {
        var error = null;
        var self = this;
        
        //Geometry errors
        if (data.errors) {
            for (var error in data.errors) {
                Giscuit.Util.jqmMessage(data.errors[error]);
            }
        }
        
        if(data.geometry) {
            //Change geometry
            
            gt.control.editgeom.editLayer.removeFeatures(this.feature, true);
            this.feature.geometry.destroy();
            this.feature.geometry = new OpenLayers.Geometry.fromWKT(data.geometry);
            gt.control.editgeom.editLayer.addFeatures(this.feature, true);
            
            this.feature.state = OpenLayers.State.UPDATE;
        }

        //Render
        var content = $("#gtEditattr > div[data-role='content']");
        content
            .off()
            .html(data.form);

         //File
         $('#gtEditattrForm').find('input[type="file"]').each(function() {
             if (! gt.control.offline.isOnline) {
                 //Set offline "delete file" buttons
                 if (self.feature.attributes[this.name]) {
                     var delBtnHtml = '<a class="file-delete" data-role="button" data-icon="delete"' +
                         ' data-iconpos="notext" title="' + gt.translate._('delete') + '"></a>';
                     $(this).after(delBtnHtml);
                 }
             }

             var delBtn = $(this).next();
             if(! delBtn.length) {
                 return;
             }
             
             delBtn.on(gt.config.clickEvt, $(this), function(evt) {
                 if(confirm(gt.translate._('deleteConfirm'))) {
                     evt.data.attr('disabled', 'disabled');
                     evt.data.prev().val('');
                 }
             });
         });
        
        //Fieldset
        $('#gtEditattrPH fieldset').attr('data-role', 'collapsible');
        $('#gtEditattrPH fieldset').each(function(index, value) {
            if($(value).find('ul.errors').length) {
                $(value).attr('data-collapsed', 'false');
            }
        });
        $('#gtEditattrPH fieldset').parent().prev().css('margin-top', '0.5em');
        $('#gtEditattrPH fieldset').on('expand', function () {
            var el = this;
            $('#gtEditattrPH fieldset').each(function(index, value) {
                if(el != value) {
                    $(value).trigger('collapse');
                }
            });
        });

        /*//Subform - Hide elements with class "gtSubformDefault"
        content.find('.gtSubformDefault').each(function(index, value) {
            var el = $(value);

            if(el.prop('tagName') == 'DD' || el.prop('tagName') == 'FIELDSSET') {
                el.prev().css('display', 'none');
            }
            el.css('display', 'none');
        });

        //Subform - Insert add buttons
        content.find('fieldset[subform=1] dl').each(function(index, value) {
            var el = $(value);

            var addImg = $('<img class="gtSubformAdd"' +
                'title="' + gt.translate._('add') + '"' +
                'src="' + gt.config.httpImagePath + '/list_add.png' + '">'
            );
            addImg.appendTo(el);
        });*/
        
        content.trigger('create');
        
        //Populate form
        if(! gt.control.offline.isOnline) {
            this.setFormValues('gtEditattrForm', this.feature.attributes);
            
            //Set hidden file fields
            var fileNames = gt.control.offline.getFileElementsNames(data.form);
            if (fileNames.length) {
                for (var i = 0; i < fileNames.length; i++) {
                    if (this.feature.attributes[fileNames[i]]) {
                        $('#gtEditattrForm input[type="hidden"][name="__fileid_' + fileNames[i] + '"]')
                            .val(this.feature.attributes[fileNames[i]]);
                    }
                }
            }
            
        }
        
        $.mobile.changePage($('#gtEditattr'), {
            'changeHash': true
        });
        
        //Trigger form render event
        gt.control.editgeom.events.triggerEvent("formrender", $('#gtEditattrForm'));

        var submited = false;
        //Submit
        $('#gtEditattrSubmit').on(gt.config.clickEvt, this, function(evt) {
            $(this).button('disable');
            submited = true;
            evt.data.submit();
            
            return false;
        });
        
        //Remove geometry if not submited
        var pageHide = function(evt) {
            if (! submited) {
                gt.control.editgeom.controls.editattrControl.unselect(evt.data.feature);
                gt.control.editgeom.clearInsertedFeatures();
            }
        };
        $('#gtEditattr').off('pagehide').on('pagehide', this, pageHide);
        
        //Cancel
        $('#gtEditattrCancel').on(gt.config.clickEvt, this, function(evt) {
            $.mobile.changePage($('#gtMap'), {
                'changeHash': true
            });
            
            return false;
        });
        
        //
        $('#gtEditattrVersionsButton').closest('.ui-btn').hide();
        
        //Change
        $('#gtEditattrVersionsSelect').on('change', this, function(evt) {
            evt.data.initialize(null, $(this).val());
        });
            
        if(! error) {
            $.mobile.hidePageLoadingMsg();
        }
    },
    
    submitOffline: function() {
        var editLayer = gt.control.editgeom.editLayer;

        var feature = this.feature;

        //Update
        if(feature.fid) {
            feature.attributes = this.getFormValues('gtEditattrForm', feature.attributes);
            if(typeof(feature.attributes.hstatus) == 'undefined') {
                feature.attributes.hstatus = 'U';
            }
            editLayer.protocol.update(feature);

        //Create
        } else {
            feature.attributes = this.getFormValues('gtEditattrForm', feature.attributes, true);
            feature.attributes.hstatus = 'C';
            editLayer.protocol.create(feature, {
                'callback': function(response) {
                    feature.fid = response.priv.result.insertId;
                }
            });
        }
        
        //Files
        var files = $('#gtEditattrForm').find('input[type="file"]');

        if (files.length) {
            var editLayer = gt.control.layers.getEditLayer();
            var filesTable = new Giscuit.Util.Db.Files({tableName: 'files_' + editLayer.id});
            var self = this;
            
            filesTable.getMinId(function(minId) {

                if (minId > 0) {
                    minId = 0;
                }
                    
                files.each(function(index, value) {
                    if (value.files.length) {
                        var hStatus = 'C';
                        minId--;
                            
                        if (feature.attributes[value.name]) {
                            //Delete previous file
                            filesTable['delete'](feature.attributes[value.name]);
                                
                            hStatus = 'U';
                        }
                            
                        //Update file id in feature attributes
                        feature.attributes[value.name] = minId;
                        
                        self.uploadOffline(editLayer.id, value.files[0], minId, hStatus);
                    } else {
                        var fileId = $('#gtEditattrForm input[type="hidden"][name="__fileid_' + value.name + '"]').val();
                        if (feature.attributes[value.name] && ! fileId) {
                            filesTable['delete'](feature.attributes[value.name]);
                            
                            feature.attributes[value.name] = '';
                        }
                    }
                });
                
                gt.control.editgeom.editLayer.protocol.update(feature);
            });
        }
        
        feature.state = null;
        gt.control.editgeom.controls.editattrControl.unselect(feature);
        
        $.mobile.changePage($('#gtMap'), {
            'changeHash': true
        });
    },
    
    submit: function() {
        if(! gt.control.offline.isOnline) {
            this.submitOffline();
            return;
        }
        
        var data = {};
        
        if(this.feature.attributes.gid) {
            var stm = '/edit/id/' + this.feature.attributes.gid;
            if (this.feature.state == OpenLayers.State.UPDATE) {
                var data = {'geometry': this.feature.geometry.toString()};
            }
        } else {
            var stm = '/add';
            var data = {'geometry': this.feature.geometry.toString()};
        }
        
        var url = gt.config.httpMobilePath + "/editgeom" + stm;
        
        //HACK: Temporary rename hidden input for checkbox
        this.renameHiddenCheckboxFields();
        
        //HACK: Enable all file elements
        $('#gtEditattrForm').find('input[type="file"]').each(function() {
            $(this).removeAttr('disabled');
        });
        
        $('#gtEditattrForm').ajaxSubmit({
            'type': 'POST',
            'url': url,
            'data': data,
            'success': this.submitSuccess,
            'error': Giscuit.Util.jqmAjaxError ,
            'dataType': 'json',
            'context': this
        }); 

    },
    
    submitSuccess: function (data) {
        //Restore hidden input for checkbox
        this.restoreHiddenCheckboxFields();
        
        if(data.success == false) {
            this.getSuccess(data);
        } else {
            //Destroy original
            if(typeof(this.feature.originalGeometry) != 'undefined') {
                this.feature.originalGeometry.destroy();
                this.feature.originalGeometry = undefined;
            }
            this.feature.state = null;
            gt.control.editgeom.controls.editattrControl.unselect(this.feature);
                            
            if(typeof(data.gid) != 'undefined') {
                //Update attributes
                this.feature.attributes = this.getFormValues('gtEditattrForm', this.feature.attributes, true);
                this.feature.attributes.gid = data.gid;
            } else {
                //Update attributes
                this.feature.attributes = this.getFormValues('gtEditattrForm', this.feature.attributes);
            }
            
            //Trigger add/edit event with feature attributes
            var featureData = {
                values: this.getFormValues('gtEditattrForm', this.feature.attributes, true),
                feature: this.feature
            };
            
            if (typeof(data.gid) != 'undefined') {
                featureData.values.gid = data.gid;
                gt.control.editgeom.events.triggerEvent("addfeature", featureData);
            } else {
                gt.control.editgeom.events.triggerEvent("editfeature", featureData);
            }
            
            gt.control.editgeom.backActivate();
            gt.control.editgeom.editLayer.removeFeatures(this.feature, true);
            gt.control.editgeom.editLayer.addFeatures(this.feature, true);
            
            $.mobile.changePage($('#gtMap'), {
                'changeHash': true
            });
            
            $('#gtEditattrSubmit').button('enable');
        }
    } ,
    
    getFormValues: function(form, attributes, overwrite) {
        if(typeof(overwrite) == 'undefined') {
            overwrite = false;
        }
        
        var values = $('#' + form).serializeArray();
        
        //Set default value for multi elements 
        var multiElements = $('#' + form + ' input[type="checkbox"],input[type="radio"],select[multiple="multiple"]');
        multiElements.each(function(index, element) {
            var addElement = true;
            for (var i = 0; i < values.length; i++) {
                if (element.name == values[i].name) {
                    addElement = false;
                    break;
                }
            }
            
            if (addElement) {
                values.push({
                    name: element.name,
                    value: ''
                });
            }
        });
        
        //Set file elements
        var fileElements = $('#' + form + ' input[type="file"]');
        fileElements.each(function(index, element) {
            var addElement = true;
            for (var key in attributes) {
                if (element.name == key) {
                    addElement = false;
                    break;
                }
            }
            
            if (addElement) {
                attributes[element.name] = '';
            }
        });
        
        for(var i=0; i < values.length; i++) {
            var input = $('#' + form + ' input[name="' + values[i].name + '"]');
            var value;

            //Checkbox
            if(input.length == 2 && 
                $(input[0]).attr('type') == 'checkbox' && 
                $(input[1]).attr('type') == 'hidden') {
                if ($(input[0]).is(':checked')) {
                    value = $(input[0]).val();
                } else {
                    value = $(input[1]).val();
                }
            //Multicheckbox
            } else if (input.length >= 2 && values[i].name.substr(values[i].name.length - 2, 2) == '[]') {
                
                value = [];
                for (var y = 0; y < input.length; y++) {
                    if ($(input[y]).is(':checked')) {
                        value.push($(input[y]).val());
                    }
                }
                value = value.join(", ");
                
            //Default
            } else {
                value = values[i].value;
            }
            
            //Multiselect
            var select = $('#' + form + ' select[name="' + values[i].name + '"][multiple="multiple"]');
            if (select.length) {
                if ($(select).val()) {
                    value = $(select).val().join(', ');
                }
            }
            
            //Normalize key
            var key = values[i].name;
            if (key.substr(key.length - 2, 2) == '[]') {
                key = key.substring(0, key.length - 2);
            } 
            
            //Set attribute value
            if(typeof(attributes[key]) != 'undefined' || overwrite) {
                attributes[key] = value;
            }
        }
        
        return attributes;
    },
    
    setFormValues: function(form, attributes) {
        $('#' + form + ' :input').each(Giscuit.Util.bind(function(index, value) {
            //Name
            var name = originalName = $(value).attr('name');
            if (originalName.substr(originalName.length - 2, 2) == '[]') {
                originalName = originalName.substring(0, originalName.length - 2);
            }
            
            //Type
            var type = $(value).attr('type');
            if(value.tagName.toLowerCase() != 'input') {
                type = value.tagName.toLowerCase();
            }
            
            //
            if(typeof(attributes[originalName]) == 'undefined') {
                return;
            }
            
            //Multicheckbox
            if (type == 'checkbox' && name != originalName) {
                if (Giscuit.Util.inArray(attributes[originalName].split(", "), $(value).val())) {
                    $(value).attr('checked', true).checkboxradio("refresh");
                }
            
            //Checkbox and radio
            } else if (type == 'checkbox' || type == 'radio') {
                if(attributes[name] == $(value).val()) {
                    $(value).attr('checked','checked').checkboxradio("refresh");
                } else {
                    $(value).removeAttr('checked').checkboxradio("refresh");
                }
            
            //Select
            } else if (type == 'select') {
                if ($(value).attr('multiple') == 'multiple') {
                    //Multiple
                    $(value).val(attributes[originalName].split(', ')).selectmenu("refresh");
                } else {
                    //Single
                    $(value).val(attributes[name]).selectmenu("refresh");
                }
            
            //Hidden
            } else if (type == 'hidden') {
                
            //Default
            }  else if (type != 'file') {
                $(value).val(attributes[name]);
            }
            
        }, this));
        
    },
    
    renameHiddenCheckboxFields: function() {
        $('#gtEditattrForm input[type="checkbox"]').each(function(index, value) {
            var name = $(value).attr('name');
            var input = $('#gtEditattrForm input[name="' + name + '"]:hidden');
            if (input.length) {
                $(input).attr('name', '__' + name + '_hidden');
            }
        });
    },
    
    restoreHiddenCheckboxFields: function() {
        $('#gtEditattrForm input[type="checkbox"]').each(function(index, value) {
            var name = $(value).attr('name');
            var input = $('#gtEditattrForm input[name="__' + name + '_hidden"]:hidden');
            if (input.length) {
                $(input).attr('name', name);
            }
        });
    },
    
    uploadOffline: function (layerId, fileData, fileId, hStatus) {
        var fileReader = new FileReader();
        
        fileReader.onload = function(file) {
            var filesTable = new Giscuit.Util.Db.Files({tableName: 'files_' + layerId});
            
            filesTable.create({
                id: fileId,
                data: file.target.result,
                name: fileData.name
            });
        };
        
        fileReader.readAsDataURL(fileData);
        
    },

    CLASS_NAME: "Giscuit.Mobile.Control.Editattr"
});

