Giscuit.Mobile.Layer.Jobs = OpenLayers.Class(OpenLayers.Layer.Vector, {
    
    defaultField: 'title',
    
    searchScale: 5000,

    initialize: function(name, options) {
        //Required
        this.visibility = true;
        
        var newArguments = [];
        newArguments.push(name, options);

        //Name
        newArguments[0] = 'Jobs';
        
        //Options
        newArguments[1] = {
            //Set layer projection
            'projection': 'EPSG:4326'
        };
        
        //Call constructor
        OpenLayers.Layer.Vector.prototype.initialize.apply(this, newArguments);
        
        //Events
        this.events.on({
            'visibilitychanged': this.load
        });
        
        var defaultStyle = OpenLayers.Util.applyDefaults({
            fillOpacity: 1 ,
            pointRadius: 7 ,
            strokeWidth: 2 ,
            fillColor: '${__getFillColor}' ,
            strokeColor: '#FFFFFF'
        }, OpenLayers.Feature.Vector.style['default']);
        
        var selectStyle = OpenLayers.Util.applyDefaults({
            fillOpacity: 1 ,
            pointRadius: 9 ,
            strokeWidth: 2 ,
            fillColor: '${__getFillColor}' ,
            strokeColor: '#FFFFFF'
        }, OpenLayers.Feature.Vector.style['select']);

        
        var style = new OpenLayers.StyleMap( {
            'default': defaultStyle,
            'select': OpenLayers.Feature.Vector.style['select']
        });

        style.styles['select'].context = style.styles['default'].context = {
            '__getFillColor': function(feature) {
                var priority = feature.attributes.priority;
                var color = '#FFFF00';
                if (priority == 'high') {
                    color = '#EE0000';
                } else if (priority == 'medium') {
                    color = '#E69138';
                } else if (priority == 'low') {
                    color = '#38761D';
                }
                    
                return color;
            }
        };

        this.styleMap = style;
        
        //Legend
        gt.control.legend.events.register("draw", this, function() {
            //Visibility
            var visible = gt.control.layers.getSettings(
                'view',
                gt.control.layers.getLayerIdByClassName(this.name)
            );
            if(! visible) {
                return;
            }
            
            if(gt.config.layout == 'desktop') {
                var panel = gt.control.legend.panel.children[1];
            } else {
                var panel = $("#gtLegend > div[data-role='content']").get(0);
            }
            
            var html = '<ul><strong>' + gt.translate._('jobs') + '</strong>';
                html += '<li style="margin-bottom: 3px;">' +
                        '<span style="background-color: #CC5050; display:inline-block; width:30px; height:20px;">&nbsp;</span> ' +
                        '<span>' + gt.translate._('priority') + ': ' + gt.translate._('high') + '</span></li>';
                html += '<li style="margin-bottom: 3px;">' +
                        '<span style="background-color: #E69138; display:inline-block; width:30px; height:20px;">&nbsp;</span> ' +
                        '<span>' + gt.translate._('priority') + ': ' + gt.translate._('medium') + '</span></li>';
                html += '<li style="margin-bottom: 3px;">' +
                        '<span style="background-color: #38761D; display:inline-block; width:30px; height:20px;">&nbsp;</span> ' +
                        '<span>' + gt.translate._('priority') + ': ' + gt.translate._('low') + '</span></li>';
            html += '</ul>';
            
            panel.innerHTML += html;
        });

        this.load();
    },
    
    load: function(evt) {
        if(! this.visibility) {
            return;
        }

        if (gt.control.offline.isOnline) {
            var url = gt.config.httpMobilePath + '/jobs/get';
            
            if (gt.control.jobs.completed) {
                url += '/completed/true/offset/' + gt.control.jobs.completedOffset;
            }
            
            $.ajax({
                'type': 'POST',
                'url': url,
                'success': this.setRecords,
                'error': Giscuit.Util.jqmAjaxError ,
                'dataType': 'json' ,
                'context': this
            });
        } else {
            this.setRecords(Giscuit.Util.getStorageObject('gtJobsRecords'));
        }

    },
    
    setRecords: function(data) {
        if (gt.control.jobs.completed) {
            if (! gt.control.jobs.completedOffset) {
                gt.control.jobs.completedRecords = data.records;
            } else {
                gt.control.jobs.completedRecords = gt.control.jobs.completedRecords.concat(data.records);
            }
            
            if (data.records.length < gt.control.jobs.completedRecordsPerRequest) {
                gt.control.jobs.loadMoreButton = false;
            }
        } else {
            Giscuit.Util.setStorageObject('gtJobsRecords', data);
            gt.control.jobs.records = data.records;
        }
        
        $.extend(true, gt.control.jobs.possibleActions, data.actions);

        //Update layer features   
        gt.control.jobs.lastSelectedFeature = null;
        this.add(data.records);
        
        //Populate jobs list
        gt.control.jobs.populate();
    },
    
    add: function(records) {        
        var newFeatures = [];
        for(var i = 0; i < records.length; i++) {
            if (! gt.control.offline.isOnline && records[i].status == 'completed') {
                continue;
            }
            if (records[i].status == 'declined') {
                continue;
            }
            
            //Geometry
            var point = new OpenLayers.Geometry.fromWKT(records[i].the_geom);
            var newFeature = new OpenLayers.Feature.Vector(point);
            
            //Parse date fields
            var date = new Date(parseFloat(records[i].date) * 1000);
            var currentDay = ( date.getDate() < 10 ? "0" : "" ) + date.getDate();
            var currentMonth = ( (date.getMonth() + 1) < 10 ? "0" : "" ) + (date.getMonth() + 1);
            var currentHours = ( date.getHours() < 10 ? "0" : "" ) + date.getHours();
            var currentMinutes = ( date.getMinutes() < 10 ? "0" : "" ) + date.getMinutes();
            var dateStr = date.getFullYear() + '-' + currentMonth + '-' + currentDay + ' ' +
                currentHours + ':' + currentMinutes;
            
            var deadline = new Date(parseFloat(records[i].deadline) * 1000);
            var currentDay = ( deadline.getDate() < 10 ? "0" : "" ) + deadline.getDate();
            var currentMonth = ( (deadline.getMonth() + 1) < 10 ? "0" : "" ) + (deadline.getMonth() + 1);
            var currentHours = ( deadline.getHours() < 10 ? "0" : "" ) + deadline.getHours();
            var currentMinutes = ( deadline.getMinutes() < 10 ? "0" : "" ) + deadline.getMinutes();
            var deadlineStr = deadline.getFullYear() + '-' + currentMonth + '-' + currentDay + ' ' +
                currentHours + ':' + currentMinutes;
            
            //Attributes
            newFeature.attributes = {
                title: records[i].title,
                desc: records[i].desc,
                contact_address: records[i].contact_address,
                contact_name: records[i].contact_name,
                contact_phone: records[i].contact_phone,
                deadline: deadlineStr,
                date: dateStr,
                status: gt.translate._(records[i].status),
                priority: records[i].priority,
                workflow: records[i].workflow
            };
            
            newFeatures.push(newFeature);
        }
        
        //Add or update elements
        this.removeAllFeatures();
        this.addFeatures(newFeatures);
    },

    CLASS_NAME: "Giscuit.Mobile.Layer.Jobs"
});
